#!/usr/bin/pythonw
"""
Find the library name for the current Python interpreter
"""
import objc
from Foundation import NSBundle


def S(*args):
    return b"".join(args)


# these are void*
NSSymbol = b"I"
NSModule = b"I"

FUNCTIONS = [
    ("NSIsSymbolNameDefined", S(objc._C_BOOL, objc._C_CHARPTR)),
    ("NSLookupAndBindSymbol", S(NSSymbol, objc._C_CHARPTR)),
    ("NSModuleForSymbol", S(NSModule, NSSymbol)),
    ("NSLibraryNameForModule", S(objc._C_CHARPTR, NSModule)),
]


def libraryNameForSymbol(symbol):
    bndl = NSBundle.bundleWithPath_("/System/Library/Frameworks/System.framework")
    d = {}
    objc.loadBundleFunctions(bndl, d, FUNCTIONS)
    for (fn, _sig) in FUNCTIONS:
        if fn not in d:
            raise ValueError(f"Couldn't find function {fn}")
    symbol = b"_" + symbol
    if not d["NSIsSymbolNameDefined"](symbol):
        # symbol not defined
        return None
    sym = d["NSLookupAndBindSymbol"](symbol)
    if not sym:
        raise ValueError(f"Couldn't bind symbol {symbol!r}")
    mod = d["NSModuleForSymbol"](sym)
    if not mod:
        raise ValueError(f"Couldn't find module for symbol {symbol!r}")
    return d["NSLibraryNameForModule"](mod)


if __name__ == "__main__":
    print(libraryNameForSymbol(b"Py_Initialize"))



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSNib(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSNibOwner, str)
        self.assertIsInstance(AppKit.NSNibTopLevelObjects, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSNib.instantiateNibWithOwner_topLevelObjects_)
        self.assertArgIsOut(AppKit.NSNib.instantiateNibWithOwner_topLevelObjects_, 1)

        self.assertResultIsBOOL(AppKit.NSNib.instantiateNibWithExternalNameTable_)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSNib.instantiateWithOwner_topLevelObjects_)
        self.assertArgIsOut(AppKit.NSNib.instantiateWithOwner_topLevelObjects_, 1)



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestNSUserActivity(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(Foundation.NSUserActivityTypeBrowsingWeb, str)

    @min_os_level("10.10")
    @expectedFailure  # XXX
    def testConstants10_10_failon1015beta(self):
        self.assertIsInstance(Foundation.NSUserActivityDocumentURLKey, str)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(Foundation.NSUserActivity.needsSave)
        self.assertArgIsBOOL(Foundation.NSUserActivity.setNeedsSave_, 0)

        self.assertResultIsBOOL(Foundation.NSUserActivity.supportsContinuationStreams)
        self.assertArgIsBOOL(
            Foundation.NSUserActivity.setSupportsContinuationStreams_, 0
        )

        self.assertArgIsBlock(
            Foundation.NSUserActivity.getContinuationStreamsWithCompletionHandler_,
            0,
            b"v@@@",
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(Foundation.NSUserActivity.isEligibleForHandoff)
        self.assertArgIsBOOL(Foundation.NSUserActivity.setEligibleForHandoff_, 0)

        self.assertResultIsBOOL(Foundation.NSUserActivity.isEligibleForSearch)
        self.assertArgIsBOOL(Foundation.NSUserActivity.setEligibleForSearch_, 0)

        self.assertResultIsBOOL(Foundation.NSUserActivity.isEligibleForPublicIndexing)
        self.assertArgIsBOOL(Foundation.NSUserActivity.setEligibleForPublicIndexing_, 0)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertArgIsBlock(
            Foundation.NSUserActivity.deleteSavedUserActivitiesWithPersistentIdentifiers_completionHandler_,
            1,
            b"v",
        )
        self.assertArgIsBlock(
            Foundation.NSUserActivity.deleteAllSavedUserActivitiesWithCompletionHandler_,
            0,
            b"v",
        )

    @min_os_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSUserActivityDelegate")



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSLayoutContraintManual(TestCase):
    def testNSDictionaryOfVariableBindings(self):
        var1 = "foo"  # noqa: F841
        var2 = "bar"  # noqa: F841

        self.assertEqual(
            AppKit.NSDictionaryOfVariableBindings("var1", "var2"),
            {"var1": "foo", "var2": "bar"},
        )

        self.assertRaises(
            KeyError, AppKit.NSDictionaryOfVariableBindings, "var1", "var3"
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(AppKit.NSViewNoIntrinsicMetric, float)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSViewNoInstrinsicMetric, float)

        self.assertEqual(AppKit.NSLayoutRelationLessThanOrEqual, -1)
        self.assertEqual(AppKit.NSLayoutRelationEqual, 0)
        self.assertEqual(AppKit.NSLayoutRelationGreaterThanOrEqual, 1)

        self.assertEqual(AppKit.NSLayoutAttributeLeft, 1)
        self.assertEqual(AppKit.NSLayoutAttributeRight, 2)
        self.assertEqual(AppKit.NSLayoutAttributeTop, 3)
        self.assertEqual(AppKit.NSLayoutAttributeBottom, 4)
        self.assertEqual(AppKit.NSLayoutAttributeLeading, 5)
        self.assertEqual(AppKit.NSLayoutAttributeTrailing, 6)
        self.assertEqual(AppKit.NSLayoutAttributeWidth, 7)
        self.assertEqual(AppKit.NSLayoutAttributeHeight, 8)
        self.assertEqual(AppKit.NSLayoutAttributeCenterX, 9)
        self.assertEqual(AppKit.NSLayoutAttributeCenterY, 10)
        self.assertEqual(AppKit.NSLayoutAttributeBaseline, 11)
        self.assertEqual(
            AppKit.NSLayoutAttributeLastBaseline, AppKit.NSLayoutAttributeBaseline
        )
        self.assertEqual(AppKit.NSLayoutAttributeFirstBaseline, 12)
        self.assertEqual(AppKit.NSLayoutAttributeNotAnAttribute, 0)

        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllLeft, (1 << AppKit.NSLayoutAttributeLeft)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllRight, (1 << AppKit.NSLayoutAttributeRight)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllTop, (1 << AppKit.NSLayoutAttributeTop)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllBottom, (1 << AppKit.NSLayoutAttributeBottom)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllLeading, (1 << AppKit.NSLayoutAttributeLeading)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllTrailing,
            (1 << AppKit.NSLayoutAttributeTrailing),
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllCenterX, (1 << AppKit.NSLayoutAttributeCenterX)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllCenterY, (1 << AppKit.NSLayoutAttributeCenterY)
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllBaseline,
            (1 << AppKit.NSLayoutAttributeBaseline),
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllLastBaseline,
            (1 << AppKit.NSLayoutAttributeBaseline),
        )
        self.assertEqual(
            AppKit.NSLayoutFormatAlignAllFirstBaseline,
            (1 << AppKit.NSLayoutAttributeFirstBaseline),
        )

        self.assertEqual(AppKit.NSLayoutFormatAlignmentMask, 0xFFFF)

        self.assertEqual(AppKit.NSLayoutFormatDirectionLeadingToTrailing, 0 << 16)
        self.assertEqual(AppKit.NSLayoutFormatDirectionLeftToRight, 1 << 16)
        self.assertEqual(AppKit.NSLayoutFormatDirectionRightToLeft, 2 << 16)

        self.assertEqual(AppKit.NSLayoutFormatDirectionMask, 0x3 << 16)

        self.assertEqual(AppKit.NSLayoutConstraintOrientationHorizontal, 0)
        self.assertEqual(AppKit.NSLayoutConstraintOrientationVertical, 1)

        self.assertEqual(AppKit.NSLayoutPriorityRequired, 1000)
        self.assertEqual(AppKit.NSLayoutPriorityDefaultHigh, 750)
        self.assertEqual(AppKit.NSLayoutPriorityDragThatCanResizeWindow, 510)
        self.assertEqual(AppKit.NSLayoutPriorityWindowSizeStayPut, 500)
        self.assertEqual(AppKit.NSLayoutPriorityDragThatCannotResizeWindow, 490)
        self.assertEqual(AppKit.NSLayoutPriorityDefaultLow, 250)
        self.assertEqual(AppKit.NSLayoutPriorityFittingSizeCompression, 50)

    @min_os_level("10.7")
    def testRecords10_7(self):
        v = AppKit.NSEdgeInsets()
        self.assertEqual(v.top, 0.0)
        self.assertEqual(v.left, 0.0)
        self.assertEqual(v.bottom, 0.0)
        self.assertEqual(v.right, 0.0)

        self.assertEqual(
            AppKit.NSEdgeInsets.__typestr__,
            b"{NSEdgeInsets="
            + objc._C_CGFloat
            + objc._C_CGFloat
            + objc._C_CGFloat
            + objc._C_CGFloat
            + b"}",
        )

    @min_os_level("10.7")
    def testFunctions10_7(self):
        v = AppKit.NSEdgeInsetsMake(1, 2, 3, 4)
        self.assertIsInstance(v, AppKit.NSEdgeInsets)
        self.assertEqual(v.top, 1.0)
        self.assertEqual(v.left, 2.0)
        self.assertEqual(v.bottom, 3.0)
        self.assertEqual(v.right, 4.0)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSLayoutConstraint.shouldBeArchived)
        self.assertArgIsBOOL(AppKit.NSLayoutConstraint.setShouldBeArchived_, 0)

        self.assertResultIsBOOL(AppKit.NSView.needsUpdateConstraints)
        self.assertArgIsBOOL(AppKit.NSView.setNeedsUpdateConstraints_, 0)

        self.assertResultIsBOOL(AppKit.NSView.needsLayout)
        self.assertArgIsBOOL(AppKit.NSView.setNeedsLayout_, 0)

        self.assertResultIsBOOL(AppKit.NSView.translatesAutoresizingMaskIntoConstraints)
        self.assertArgIsBOOL(
            AppKit.NSView.setTranslatesAutoresizingMaskIntoConstraints_, 0
        )

        self.assertResultIsBOOL(AppKit.NSView.requiresConstraintBasedLayout)
        self.assertResultIsBOOL(AppKit.NSView.hasAmbiguousLayout)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSLayoutConstraint.isActive)
        self.assertArgIsBOOL(AppKit.NSLayoutConstraint.setActive_, 0)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(AppKit.NSView.isHorizontalContentSizeConstraintActive)
        self.assertArgIsBOOL(AppKit.NSView.setHorizontalContentSizeConstraintActive_, 0)

        self.assertResultIsBOOL(AppKit.NSView.isVerticalContentSizeConstraintActive)
        self.assertArgIsBOOL(AppKit.NSView.setVerticalContentSizeConstraintActive_, 0)



import array
import os

import CoreFoundation
from Foundation import NSURL
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestURL(TestCase):
    def testTypes(self):
        self.assertIs(CoreFoundation.CFURLRef, NSURL)

    def testTypeID(self):
        val = CoreFoundation.CFURLGetTypeID()
        self.assertIsInstance(val, int)

    def testCreateWithBytes(self):
        url = b"http://www.omroep.nl/"

        ref = CoreFoundation.CFURLCreateWithBytes(
            None, url, len(url), CoreFoundation.kCFStringEncodingUTF8, None
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        strval = CoreFoundation.CFURLGetString(ref)
        self.assertEqual(strval, str(url, "utf-8"))

        ref2 = CoreFoundation.CFURLCreateWithBytes(
            None, url, len(url), CoreFoundation.kCFStringEncodingUTF8, ref
        )
        self.assertIsInstance(ref2, CoreFoundation.CFURLRef)

        a = array.array("b", b"http://www.nu.nl/")
        ref3 = CoreFoundation.CFURLCreateWithBytes(
            None, a, len(a), CoreFoundation.kCFStringEncodingUTF8, None
        )
        self.assertIsInstance(ref3, CoreFoundation.CFURLRef)

        # Explictely test for str's buffer madness.
        self.assertRaises(
            (ValueError, TypeError),
            CoreFoundation.CFURLCreateWithBytes,
            None,
            str(url),
            len(url),
            CoreFoundation.kCFStringEncodingUTF8,
            None,
        )

    def testCreateData(self):
        url = b"http://www.omroep.nl/ blank"

        ref = CoreFoundation.CFURLCreateWithBytes(
            None, url, len(url), CoreFoundation.kCFStringEncodingUTF8, None
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        data = CoreFoundation.CFURLCreateData(
            None, ref, CoreFoundation.kCFStringEncodingUTF8, False
        )
        self.assertIsInstance(data, CoreFoundation.CFDataRef)
        val = CoreFoundation.CFDataGetBytes(
            data, (0, CoreFoundation.CFDataGetLength(data)), None
        )
        self.assertEqual(val, url.replace(b" ", b"%20"))

        data = CoreFoundation.CFURLCreateData(
            None, ref, CoreFoundation.kCFStringEncodingUTF8, True
        )
        self.assertIsInstance(data, CoreFoundation.CFDataRef)
        val = CoreFoundation.CFDataGetBytes(
            data, (0, CoreFoundation.CFDataGetLength(data)), None
        )
        self.assertEqual(val, url.replace(b" ", b"%20"))

    def testCreateWithString(self):
        url = "http://www.omroep.nl/"

        ref = CoreFoundation.CFURLCreateWithString(None, url, None)
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        strval = CoreFoundation.CFURLGetString(ref)
        self.assertEqual(strval, url)

        ref2 = CoreFoundation.CFURLCreateWithString(None, url, ref)
        self.assertIsInstance(ref2, CoreFoundation.CFURLRef)

    def testCreateAbsolute(self):
        url = "http://www.omroep.nl/sport/"
        baseref = CoreFoundation.CFURLCreateWithString(None, url, None)

        self.assertArgHasType(CoreFoundation.CFURLCreateAbsoluteURLWithBytes, 1, b"n^v")
        self.assertArgSizeInArg(CoreFoundation.CFURLCreateAbsoluteURLWithBytes, 1, 2)
        ref = CoreFoundation.CFURLCreateAbsoluteURLWithBytes(
            None,
            b"socker",
            len(b"socker"),
            CoreFoundation.kCFStringEncodingUTF8,
            baseref,
            True,
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        strval = CoreFoundation.CFURLGetString(ref)
        self.assertEqual(strval, "http://www.omroep.nl/sport/socker")

        relpath = b"../../../dummy"
        ref = CoreFoundation.CFURLCreateAbsoluteURLWithBytes(
            None,
            relpath,
            len(relpath),
            CoreFoundation.kCFStringEncodingUTF8,
            baseref,
            True,
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(ref)
        self.assertEqual(strval, "http://www.omroep.nl/dummy")

        relpath = b"../../../dummy"
        ref = CoreFoundation.CFURLCreateAbsoluteURLWithBytes(
            None,
            relpath,
            len(relpath),
            CoreFoundation.kCFStringEncodingUTF8,
            baseref,
            False,
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(ref)
        self.assertEqual(strval, "http://www.omroep.nl/../../dummy")

    def testCopyAbs(self):
        # CoreFoundation.CFURLCopyAbsoluteURL
        base = CoreFoundation.CFURLCreateWithString(None, "http://www.omroep.nl/", None)
        self.assertIsInstance(base, CoreFoundation.CFURLRef)

        ref = CoreFoundation.CFURLCreateWithString(None, "/sport", base)
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        self.assertEqual(CoreFoundation.CFURLGetString(ref), "/sport")
        abs_url = CoreFoundation.CFURLCopyAbsoluteURL(ref)
        self.assertIsInstance(abs_url, CoreFoundation.CFURLRef)
        self.assertEqual(
            CoreFoundation.CFURLGetString(abs_url), "http://www.omroep.nl/sport"
        )

    def testPaths(self):
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, "/tmp/", CoreFoundation.kCFURLPOSIXPathStyle, True
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        self.assertTrue(CoreFoundation.CFURLHasDirectoryPath(url))

        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, "/etc/hosts", CoreFoundation.kCFURLPOSIXPathStyle, False
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        self.assertFalse(CoreFoundation.CFURLHasDirectoryPath(url))

        p = os.path.expanduser("~")
        p = p.encode("utf-8")
        self.assertArgHasType(
            CoreFoundation.CFURLCreateFromFileSystemRepresentation, 1, b"n^t"
        )
        self.assertArgIsNullTerminated(
            CoreFoundation.CFURLCreateFromFileSystemRepresentation, 1
        )
        url = CoreFoundation.CFURLCreateFromFileSystemRepresentation(
            None, p, len(p), True
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        self.assertRaises(
            (ValueError, TypeError),
            CoreFoundation.CFURLCreateFromFileSystemRepresentation,
            None,
            "/tmp/",
            4,
            True,
        )

        base = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, "/tmp", CoreFoundation.kCFURLPOSIXPathStyle, True
        )
        self.assertIsInstance(base, CoreFoundation.CFURLRef)

        self.assertArgIsBOOL(
            CoreFoundation.CFURLCreateWithFileSystemPathRelativeToBase, 3
        )
        url = CoreFoundation.CFURLCreateWithFileSystemPathRelativeToBase(
            None, "filename", CoreFoundation.kCFURLPOSIXPathStyle, True, base
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)

        strval = CoreFoundation.CFURLGetString(url)
        self.assertEqual(strval, "filename/")

        self.assertArgIsBOOL(
            CoreFoundation.CFURLCreateFromFileSystemRepresentationRelativeToBase, 3
        )
        url = CoreFoundation.CFURLCreateFromFileSystemRepresentationRelativeToBase(
            None, b"filename2", 9, False, base
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(url)
        self.assertEqual(strval, "filename2")

        ok, strval = CoreFoundation.CFURLGetFileSystemRepresentation(
            url, True, None, 100
        )
        self.assertTrue(ok)

        # Unfortunately metadata doesn't allow describing what we actually need
        if b"\0" in strval:
            strval = strval[: strval.index(b"\0")]
        self.assertEqual(strval, b"/tmp/filename2")

    def testParts(self):
        base = CoreFoundation.CFURLCreateWithString(None, "http://www.omroep.nl/", None)
        self.assertIsInstance(base, CoreFoundation.CFURLRef)

        ref = CoreFoundation.CFURLCreateWithString(None, "/sport", base)
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        self.assertEqual(CoreFoundation.CFURLGetBaseURL(base), None)
        self.assertEqual(CoreFoundation.CFURLGetBaseURL(ref), base)
        self.assertTrue(CoreFoundation.CFURLCanBeDecomposed(ref) is True)

        self.assertEqual(CoreFoundation.CFURLCopyScheme(ref), "http")
        self.assertEqual(CoreFoundation.CFURLCopyNetLocation(ref), "www.omroep.nl")
        self.assertEqual(CoreFoundation.CFURLCopyPath(ref), "/sport")

        path, isDir = CoreFoundation.CFURLCopyStrictPath(ref, None)
        self.assertEqual(path, "sport")
        self.assertEqual(isDir, True)

        path = CoreFoundation.CFURLCopyFileSystemPath(
            ref, CoreFoundation.kCFURLPOSIXPathStyle
        )
        self.assertEqual(path, "/sport")

        path = CoreFoundation.CFURLCopyFileSystemPath(
            ref, CoreFoundation.kCFURLPOSIXPathStyle
        )
        self.assertEqual(path, "/sport")
        path = CoreFoundation.CFURLCopyFileSystemPath(
            ref, CoreFoundation.kCFURLWindowsPathStyle
        )
        self.assertEqual(path, "\\sport")

        self.assertFalse(CoreFoundation.CFURLHasDirectoryPath(ref))

        v = CoreFoundation.CFURLCopyResourceSpecifier(ref)
        self.assertEqual(v, None)

        v = CoreFoundation.CFURLCopyHostName(ref)
        self.assertEqual(v, "www.omroep.nl")

        v = CoreFoundation.CFURLGetPortNumber(ref)
        self.assertEqual(v, -1)

        ref = CoreFoundation.CFURLCreateWithString(
            None,
            b"https://ronald:test@www.nu.nl:42/sport/results.cgi?qs=1#anchor".decode(
                "ascii"
            ),
            None,
        )
        v = CoreFoundation.CFURLGetPortNumber(ref)
        self.assertEqual(v, 42)

        v = CoreFoundation.CFURLCopyResourceSpecifier(ref)
        self.assertEqual(v, "?qs=1#anchor")

        v = CoreFoundation.CFURLCopyUserName(ref)
        self.assertEqual(v, "ronald")

        v = CoreFoundation.CFURLCopyPassword(ref)
        self.assertEqual(v, "test")

        v = CoreFoundation.CFURLCopyParameterString(ref, None)
        self.assertEqual(v, None)

        v = CoreFoundation.CFURLCopyQueryString(ref, None)
        self.assertEqual(v, "qs=1")

        v = CoreFoundation.CFURLCopyLastPathComponent(ref)
        self.assertEqual(v, "results.cgi")

        v = CoreFoundation.CFURLCopyPathExtension(ref)
        self.assertEqual(v, "cgi")

        cnt, bytes_value = CoreFoundation.CFURLGetBytes(ref, None, 100)
        self.assertEqual(cnt, 62)
        self.assertEqual(
            bytes_value,
            b"https://ronald:test@www.nu.nl:42/sport/results.cgi?qs=1#anchor",
        )

        cnt, bytes_value = CoreFoundation.CFURLGetBytes(ref, objc.NULL, 0)
        self.assertEqual(cnt, 62)
        self.assertEqual(bytes_value, objc.NULL)

        rng1, rng2 = CoreFoundation.CFURLGetByteRangeForComponent(
            ref, CoreFoundation.kCFURLComponentHost, None
        )
        self.assertIsInstance(rng1, CoreFoundation.CFRange)
        self.assertIsInstance(rng2, CoreFoundation.CFRange)

    def testUpdating(self):
        base = CoreFoundation.CFURLCreateWithString(
            None, "http://www.omroep.nl/sport", None
        )
        self.assertIsInstance(base, CoreFoundation.CFURLRef)

        url = CoreFoundation.CFURLCreateCopyAppendingPathComponent(
            None, base, "soccer", True
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)

        strval = CoreFoundation.CFURLGetString(url)
        self.assertEqual(strval, "http://www.omroep.nl/sport/soccer/")

        url = CoreFoundation.CFURLCreateCopyDeletingLastPathComponent(None, base)
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(url)
        self.assertEqual(strval, "http://www.omroep.nl/")

        url = CoreFoundation.CFURLCreateCopyAppendingPathExtension(None, base, "cgi")
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(url)
        self.assertEqual(strval, "http://www.omroep.nl/sport.cgi")

        url2 = CoreFoundation.CFURLCreateCopyDeletingPathExtension(None, base)
        self.assertIsInstance(url2, CoreFoundation.CFURLRef)
        strval = CoreFoundation.CFURLGetString(url2)
        self.assertEqual(strval, "http://www.omroep.nl/sport")

    def testStringEncoding(self):
        base = "http://www.omroep.nl/sport%20en%20%73%70el"

        strval = CoreFoundation.CFURLCreateStringByReplacingPercentEscapes(
            None, base, objc.NULL
        )
        self.assertEqual(strval, "http://www.omroep.nl/sport%20en%20%73%70el")

        strval = CoreFoundation.CFURLCreateStringByReplacingPercentEscapes(
            None, base, ""
        )
        self.assertEqual(strval, "http://www.omroep.nl/sport en spel")

        strval = CoreFoundation.CFURLCreateStringByReplacingPercentEscapes(
            None, base, " "
        )
        self.assertEqual(strval, "http://www.omroep.nl/sport%20en%20spel")

        strval = CoreFoundation.CFURLCreateStringByReplacingPercentEscapesUsingEncoding(
            None, base, "", CoreFoundation.kCFStringEncodingISOLatin1
        )
        self.assertEqual(strval, "http://www.omroep.nl/sport en spel")

        base = "http://www.omroep.nl/sport en spel"
        strval = CoreFoundation.CFURLCreateStringByAddingPercentEscapes(
            None, base, "", "", CoreFoundation.kCFStringEncodingISOLatin1
        )
        self.assertEqual(strval, "http://www.omroep.nl/sport%20en%20spel")
        strval = CoreFoundation.CFURLCreateStringByAddingPercentEscapes(
            None, base, " ", "s", CoreFoundation.kCFStringEncodingISOLatin1
        )
        self.assertEqual(strval, "http://www.omroep.nl/%73port en %73pel")

    def testFSRef(self):
        ref = CoreFoundation.CFURLCreateWithFileSystemPath(
            None, os.getcwd(), CoreFoundation.kCFURLPOSIXPathStyle, True
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLRef)

        ok, fsref = CoreFoundation.CFURLGetFSRef(ref, None)
        self.assertTrue(ok)
        self.assertIsInstance(fsref, objc.FSRef)
        self.assertEqual(fsref.as_pathname(), os.getcwd())

        ref2 = CoreFoundation.CFURLCreateFromFSRef(None, fsref)
        self.assertEqual(ref, ref2)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFURLPOSIXPathStyle, 0)
        self.assertEqual(CoreFoundation.kCFURLHFSPathStyle, 1)
        self.assertEqual(CoreFoundation.kCFURLWindowsPathStyle, 2)

        self.assertEqual(CoreFoundation.kCFURLComponentScheme, 1)
        self.assertEqual(CoreFoundation.kCFURLComponentNetLocation, 2)
        self.assertEqual(CoreFoundation.kCFURLComponentPath, 3)
        self.assertEqual(CoreFoundation.kCFURLComponentResourceSpecifier, 4)
        self.assertEqual(CoreFoundation.kCFURLComponentUser, 5)
        self.assertEqual(CoreFoundation.kCFURLComponentPassword, 6)
        self.assertEqual(CoreFoundation.kCFURLComponentUserInfo, 7)
        self.assertEqual(CoreFoundation.kCFURLComponentHost, 8)
        self.assertEqual(CoreFoundation.kCFURLComponentPort, 9)
        self.assertEqual(CoreFoundation.kCFURLComponentParameterString, 10)
        self.assertEqual(CoreFoundation.kCFURLComponentQuery, 11)
        self.assertEqual(CoreFoundation.kCFURLComponentFragment, 12)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        fp = open("/tmp/pyobjc.test", "w")
        fp.close()
        try:
            baseURL = CoreFoundation.CFURLCreateWithFileSystemPath(
                None,
                os.path.realpath("/tmp/pyobjc.test"),
                CoreFoundation.kCFURLPOSIXPathStyle,
                False,
            )
            self.assertIsInstance(baseURL, CoreFoundation.CFURLRef)

            self.assertResultIsCFRetained(CoreFoundation.CFURLCreateFileReferenceURL)
            url, err = CoreFoundation.CFURLCreateFileReferenceURL(None, baseURL, None)
            self.assertIsInstance(url, CoreFoundation.CFURLRef)
            self.assertEqual(err, None)

            self.assertResultIsCFRetained(CoreFoundation.CFURLCreateFilePathURL)
            url, err = CoreFoundation.CFURLCreateFilePathURL(None, baseURL, None)
            self.assertIsInstance(url, CoreFoundation.CFURLRef)
            self.assertEqual(err, None)

            self.assertResultIsBOOL(CoreFoundation.CFURLCopyResourcePropertyForKey)
            self.assertArgIsCFRetained(
                CoreFoundation.CFURLCopyResourcePropertyForKey, 2
            )
            self.assertArgIsOut(CoreFoundation.CFURLCopyResourcePropertyForKey, 2)
            self.assertArgIsOut(CoreFoundation.CFURLCopyResourcePropertyForKey, 3)
            ok, value, error = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, CoreFoundation.kCFURLNameKey, None, None
            )
            self.assertTrue(ok)
            self.assertIsInstance(value, str)
            self.assertEqual(error, None)

            ok, value, error = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, CoreFoundation.kCFURLIsRegularFileKey, None, None
            )
            self.assertTrue(ok)
            self.assertIsInstance(value, bool)
            self.assertEqual(error, None)

            self.assertResultIsCFRetained(CoreFoundation.CFURLCreateFilePathURL)
            self.assertArgIsOut(CoreFoundation.CFURLCopyResourcePropertyForKey, 2)
            values, error = CoreFoundation.CFURLCopyResourcePropertiesForKeys(
                url,
                [CoreFoundation.kCFURLNameKey, CoreFoundation.kCFURLIsRegularFileKey],
                None,
            )
            self.assertIsInstance(values, CoreFoundation.CFDictionaryRef)
            self.assertEqual(error, None)

            CoreFoundation.CFURLClearResourcePropertyCacheForKey(
                url, CoreFoundation.kCFURLIsRegularFileKey
            )
            CoreFoundation.CFURLClearResourcePropertyCache(url)
            self.assertResultIsBOOL(CoreFoundation.CFURLResourceIsReachable)
            v, err = CoreFoundation.CFURLResourceIsReachable(url, None)
            self.assertIsInstance(v, bool)
            self.assertEqual(err, None)

            CoreFoundation.CFURLSetTemporaryResourcePropertyForKey(
                url, "pyobjc.test", "hello"
            )
            ok, v, err = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, "pyobjc.test", None, None
            )
            self.assertTrue(ok)
            self.assertEqual(v, "hello")

            ok, cur, err = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, CoreFoundation.kCFURLIsHiddenKey, None, None
            )
            self.assertTrue(ok)

            ok, err = CoreFoundation.CFURLSetResourcePropertyForKey(
                url, CoreFoundation.kCFURLIsHiddenKey, not cur, None
            )
            self.assertTrue(ok)

            ok, new, err = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, CoreFoundation.kCFURLIsHiddenKey, None, None
            )
            self.assertTrue(ok)
            self.assertEqual(new, not cur)
            self.assertEqual(err, None)

            ok, err = CoreFoundation.CFURLSetResourcePropertiesForKeys(
                url, {CoreFoundation.kCFURLIsHiddenKey: cur}, None
            )
            self.assertTrue(ok)
            self.assertEqual(err, None)

            ok, new, err = CoreFoundation.CFURLCopyResourcePropertyForKey(
                url, CoreFoundation.kCFURLIsHiddenKey, None, None
            )
            self.assertTrue(ok)
            self.assertEqual(new, cur)
            self.assertEqual(err, None)

            self.assertResultIsCFRetained(CoreFoundation.CFURLCreateBookmarkData)
            data, err = CoreFoundation.CFURLCreateBookmarkData(
                None,
                url,
                CoreFoundation.kCFURLBookmarkCreationSuitableForBookmarkFile,
                [CoreFoundation.kCFURLNameKey, CoreFoundation.kCFURLIsHiddenKey],
                None,
                None,
            )
            self.assertIs(err, None)
            self.assertIsInstance(data, CoreFoundation.CFDataRef)

            self.assertResultIsCFRetained(
                CoreFoundation.CFURLCreateByResolvingBookmarkData
            )
            u, stale, err = CoreFoundation.CFURLCreateByResolvingBookmarkData(
                None, data, 0, None, None, None, None
            )
            self.assertEqual(u, url)
            self.assertIsInstance(stale, bool)
            self.assertFalse(stale)
            self.assertIs(err, None)
            self.assertResultIsCFRetained(
                CoreFoundation.CFURLCreateResourcePropertiesForKeysFromBookmarkData
            )
            v = CoreFoundation.CFURLCreateResourcePropertiesForKeysFromBookmarkData(
                None, [CoreFoundation.kCFURLNameKey], data
            )
            self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)

            self.assertResultIsCFRetained(
                CoreFoundation.CFURLCreateResourcePropertyForKeyFromBookmarkData
            )
            v = CoreFoundation.CFURLCreateResourcePropertyForKeyFromBookmarkData(
                None, CoreFoundation.kCFURLNameKey, data
            )
            self.assertIsInstance(v, str)

            refURL = CoreFoundation.CFURLCreateWithFileSystemPath(
                None, "/tmp/pyobjc.test.2", CoreFoundation.kCFURLPOSIXPathStyle, False
            )
            ok, err = CoreFoundation.CFURLWriteBookmarkDataToFile(data, refURL, 0, None)
            self.assertTrue(ok)
            self.assertIs(err, None)
            self.assertTrue(os.path.exists("/tmp/pyobjc.test.2"))

            self.assertResultIsCFRetained(
                CoreFoundation.CFURLCreateBookmarkDataFromFile
            )
            n, err = CoreFoundation.CFURLCreateBookmarkDataFromFile(None, refURL, None)
            self.assertIsInstance(n, CoreFoundation.CFDataRef)
            self.assertIs(err, None)
            self.assertResultIsCFRetained(
                CoreFoundation.CFURLCreateBookmarkDataFromAliasRecord
            )
            self.assertArgHasType(
                CoreFoundation.CFURLCreateBookmarkDataFromAliasRecord,
                0,
                b"^{__CFAllocator=}",
            )
            self.assertArgHasType(
                CoreFoundation.CFURLCreateBookmarkDataFromAliasRecord,
                1,
                b"^{__CFData=}",
            )

        finally:
            os.unlink("/tmp/pyobjc.test")
            if os.path.exists("/tmp/pyobjc.test.2"):
                os.unlink("/tmp/pyobjc.test.2")

    @min_os_level("10.8")
    def testFunctions10_8(self):
        self.assertResultIsBOOL(
            CoreFoundation.CFURLStartAccessingSecurityScopedResource
        )
        CoreFoundation.CFURLStopAccessingSecurityScopedResource

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(CoreFoundation.kCFURLIsExcludedFromBackupKey, str)

        self.assertIsInstance(CoreFoundation.kCFURLPathKey, str)

        self.assertEqual(CoreFoundation.kCFBookmarkResolutionWithoutUIMask, 1 << 8)
        self.assertEqual(
            CoreFoundation.kCFBookmarkResolutionWithoutMountingMask, 1 << 9
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkResolutionWithSecurityScope, 1 << 10
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkResolutionWithoutImplicitStartAccessing,
            1 << 15,
        )

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(CoreFoundation.kCFURLTagNamesKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemDownloadingStatusKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemDownloadingErrorKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLUbiquitousItemUploadingErrorKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemDownloadingStatusNotDownloaded, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemDownloadingStatusDownloaded, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemDownloadingStatusCurrent, str
        )

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(CoreFoundation.kCFURLGenerationIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLDocumentIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLAddedToDirectoryDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLQuarantinePropertiesKey, str)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(CoreFoundation.kCFURLIsApplicationKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLApplicationIsScriptableKey, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(CoreFoundation.kCFURLNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLocalizedNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsRegularFileKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsDirectoryKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsSymbolicLinkKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsVolumeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsPackageKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsSystemImmutableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsUserImmutableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsHiddenKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLHasHiddenExtensionKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLCreationDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLContentAccessDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLContentModificationDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLAttributeModificationDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLinkCountKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLParentDirectoryURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLTypeIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLocalizedTypeDescriptionKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLabelNumberKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLabelColorKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLocalizedLabelKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLEffectiveIconKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLCustomIconKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileAllocatedSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsAliasFileKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeLocalizedFormatDescriptionKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLVolumeTotalCapacityKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeAvailableCapacityKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeResourceCountKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsPersistentIDsKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsSymbolicLinksKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsHardLinksKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsJournalingKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsJournalingKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsSparseFilesKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsZeroRunsKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsCaseSensitiveNamesKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsCasePreservedNamesKey, str
        )

        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationPreferFileIDResolutionMask, 1 << 8
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationMinimalBookmarkMask, 1 << 9
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationSuitableForBookmarkFile, 1 << 10
        )
        self.assertEqual(CoreFoundation.kCFBookmarkResolutionWithoutUIMask, 1 << 8)
        self.assertEqual(
            CoreFoundation.kCFBookmarkResolutionWithoutMountingMask, 1 << 9
        )

        self.assertEqual(CoreFoundation.kCFURLBookmarkResolutionWithoutUIMask, 1 << 8)
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkResolutionWithoutMountingMask, 1 << 9
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationWithSecurityScope, 1 << 11
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationSecurityScopeAllowOnlyReadAccess,
            1 << 12,
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkResolutionWithSecurityScope, 1 << 10
        )
        self.assertEqual(
            CoreFoundation.kCFURLBookmarkCreationWithoutImplicitSecurityScope, 1 << 29
        )

        self.assertIsInstance(CoreFoundation.kCFURLKeysOfUnsetValuesKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLPreferredIOBlockSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsReadableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsWritableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsExecutableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileSecurityKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeNamedPipe, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLFileResourceTypeCharacterSpecial, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeDirectory, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeBlockSpecial, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeRegular, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeSymbolicLink, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeSocket, str)
        self.assertIsInstance(CoreFoundation.kCFURLFileResourceTypeUnknown, str)
        self.assertIsInstance(CoreFoundation.kCFURLTotalFileSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLTotalFileAllocatedSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsMountTriggerKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsRootDirectoryDatesKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsVolumeSizesKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsRenamingKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsAdvisoryFileLockingKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsExtendedSecurityKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsBrowsableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeMaximumFileSizeKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsEjectableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsRemovableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsInternalKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsAutomountedKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsLocalKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsReadOnlyKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeCreationDateKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeURLForRemountingKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeUUIDStringKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeLocalizedNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsUbiquitousItemKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemHasUnresolvedConflictsKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLUbiquitousItemIsDownloadedKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLUbiquitousItemIsDownloadingKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLUbiquitousItemIsUploadedKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLUbiquitousItemIsUploadingKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemPercentDownloadedKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemPercentUploadedKey, str
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(CoreFoundation.kCFURLVolumeLocalizedNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsEncryptedKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeIsRootFileSystemKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsCompressionKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsFileCloningKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsSwapRenamingKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsExclusiveRenamingKey, str
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeAvailableCapacityForImportantUsageKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeAvailableCapacityForOpportunisticUsageKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsImmutableFilesKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFURLVolumeSupportsAccessPermissionsKey, str
        )

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(CoreFoundation.kCFURLFileContentIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLMayShareFileContentKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLMayHaveExtendedAttributesKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsPurgeableKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLIsSparseKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLLinkCountKey, str)
        self.assertIsInstance(CoreFoundation.kCFURLVolumeSupportsFileProtectionKey, str)

    @min_os_level("11.3")
    def testConstants11_3(self):
        self.assertIsInstance(
            CoreFoundation.kCFURLUbiquitousItemIsExcludedFromSyncKey, str
        )

    @min_os_level("10.9")
    def testFunctions10_9(self):
        self.assertResultIsBOOL(CoreFoundation.CFURLIsFileReferenceURL)



#
#  JoystickView.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from math import atan2, cos, pi, sin, sqrt

import objc
from Cocoa import (
    NSError,
    NSLocalizedDescriptionKey,
    NSAffineTransform,
    NSBezierPath,
    NSColor,
    NSDrawDarkBezel,
    NSDrawLightBezel,
    NSInsetRect,
    NSLocalizedStringFromTable,
    NSMakePoint,
    NSMakeRect,
    NSMultipleValuesMarker,
    NSNoSelectionMarker,
    NSNotApplicableMarker,
    NSNumber,
    NSShiftKeyMask,
    NSValueTransformer,
    NSView,
)
from objc import super


class JoystickView(NSView):
    AngleObservationContext = 2091
    OffsetObservationContext = 2092

    maxOffset = objc.ivar("maxOffset", objc._C_DBL)
    angle = objc.ivar("angle")  # , 'd') # expect angle in degrees
    offset = objc.ivar("offset")  # , 'd')

    observedObjectForAngle = objc.ivar("observedObjectForAngle")
    observedKeyPathForAngle = objc.ivar("observedKeyPathForAngle")
    angleValueTransformerName = objc.ivar("angleValueTransformerName")
    badSelectionForAngle = objc.ivar("badSelectionForAngle")
    multipleSelectionForAngle = objc.ivar("multipleSelectionForAngle")
    allowsMultipleSelectionForAngle = objc.ivar("allowsMultipleSelectionForAngle")

    observedObjectForOffset = objc.ivar("observedObjectForOffset")
    observedKeyPathForOffset = objc.ivar("observedKeyPathForOffset")
    offsetValueTransformerName = objc.ivar("offsetValueTransformerName")
    badSelectionForOffset = objc.ivar("badSelectionForOffset")
    multipleSelectionForOffset = objc.ivar("multipleSelectionForOffset")
    allowsMultipleSelectionForOffset = objc.ivar("allowsMultipleSelectionForOffset")

    @classmethod
    def valueClassForBinding_(cls, binding):
        # both require numbers
        return NSNumber

    def initWithFrame_(self, frameRect):
        self = super().initWithFrame_(frameRect)
        if self is None:
            return None
        self.maxOffset = 15.0
        self.offset = 0.0
        self.angle = 28.0
        self.multipleSelectionForAngle = False
        self.multipleSelectionForOffset = False
        return self

    def bind_toObject_withKeyPath_options_(
        self, bindingName, observableController, keyPath, options
    ):

        if bindingName == "angle":
            # observe the controller for changes -- note, pass binding identifier
            # as the context, so we get that back in observeValueForKeyPath:...
            # that way we can determine what needs to be updated.
            observableController.addObserver_forKeyPath_options_context_(
                self, keyPath, 0, self.AngleObservationContext
            )
            # register what controller and what keypath are
            # associated with this binding
            self.observedObjectForAngle = observableController
            self.observedKeyPathForAngle = keyPath
            # options
            self.angleValueTransformerName = options["NSValueTransformerName"]
            self.allowsMultipleSelectionForAngle = False
            if options["NSAllowsEditingMultipleValuesSelection"]:
                self.allowsMultipleSelectionForAngle = True

        if bindingName == "offset":
            observableController.addObserver_forKeyPath_options_context_(
                self, keyPath, 0, self.OffsetObservationContext
            )
            self.observedObjectForOffset = observableController
            self.observedKeyPathForOffset = keyPath
            self.allowsMultipleSelectionForOffset = False
            if options["NSAllowsEditingMultipleValuesSelection"]:
                self.allowsMultipleSelectionForOffset = True

    def unbind_(self, bindingName):
        if bindingName == "angle":
            if self.observedObjectForAngle is None:
                return
            self.observedObjectForAngle.removeObserver_forKeyPath_(
                self, self.observedKeyPathForAngle
            )
            self.observedObjectForAngle = None
            self.observedKeyPathForAngle = None
            self.angleValueTransformerName = None
        elif bindingName == "offset":
            if self.observedObjectForOffset is None:
                return None
            self.observedObjectForOffset.removeObserver_forKeyPath_(
                self, self.observedKeyPathForOffset
            )
            self.observedObjectForOffset = None
            self.observedKeyPathForOffset = None

    def observeValueForKeyPath_ofObject_change_context_(
        self, keyPath, an_object, change, context
    ):
        # we passed the binding as the context when we added ourselves
        # as an observer -- use that to decide what to update...
        # should ask the dictionary for the value...
        if context == self.AngleObservationContext:
            # angle changed
            # if we got a NSNoSelectionMarker or NSNotApplicableMarker, or
            # if we got a NSMultipleValuesMarker and we don't allow multiple selections
            # then note we have a bad angle
            newAngle = self.observedObjectForAngle.valueForKeyPath_(
                self.observedKeyPathForAngle
            )
            if (
                newAngle == NSNoSelectionMarker
                or newAngle == NSNotApplicableMarker
                or (
                    newAngle == NSMultipleValuesMarker
                    and not self.allowsMultipleSelectionForAngle
                )
            ):
                self.badSelectionForAngle = True

            else:
                # note we have a good selection
                # if we got a NSMultipleValuesMarker, note it but don't update value
                self.badSelectionForAngle = False
                if newAngle == NSMultipleValuesMarker:
                    self.multipleSelectionForAngle = True
                else:
                    self.multipleSelectionForAngle = False
                    if self.angleValueTransformerName is not None:
                        vt = NSValueTransformer.valueTransformerForName_(
                            self.angleValueTransformerName
                        )
                        newAngle = vt.transformedValue_(newAngle)
                    self.setValue_forKey_(newAngle, "angle")

        if context == self.OffsetObservationContext:
            # offset changed
            # if we got a NSNoSelectionMarker or NSNotApplicableMarker, or
            # if we got a NSMultipleValuesMarker and we don't allow multiple selections
            # then note we have a bad selection
            newOffset = self.observedObjectForOffset.valueForKeyPath_(
                self.observedKeyPathForOffset
            )
            if (
                newOffset == NSNoSelectionMarker
                or newOffset == NSNotApplicableMarker
                or (
                    newOffset == NSMultipleValuesMarker
                    and not self.allowsMultipleSelectionForOffset
                )
            ):
                self.badSelectionForOffset = True
            else:
                # note we have a good selection
                # if we got a NSMultipleValuesMarker, note it but don't update value
                self.badSelectionForOffset = False
                if newOffset == NSMultipleValuesMarker:
                    self.multipleSelectionForOffset = True
                else:
                    self.setValue_forKey_(newOffset, "offset")
                    self.multipleSelectionForOffset = False
        self.setNeedsDisplay_(True)

    def updateForMouseEvent_(self, event):
        """
        update based on event location and selection state
        behavior based on modifier key
        """
        if self.badSelectionForAngle or self.badSelectionForOffset:
            return  # don't do anything

        # find out where the event is, offset from the view center
        p = self.convertPoint_fromView_(event.locationInWindow(), None)
        myBounds = self.bounds()
        xOffset = p.x - (myBounds.size.width / 2)
        yOffset = p.y - (myBounds.size.height / 2)

        newOffset = sqrt(xOffset * xOffset + yOffset * yOffset)
        if newOffset > self.maxOffset:
            newOffset = self.maxOffset
        elif newOffset < -self.maxOffset:
            newOffset = -self.maxOffset

        # if we have a multiple selection for offset and Shift key is pressed
        # then don't update the offset
        # this allows offsets to remain constant, but change angle
        if not (
            self.multipleSelectionForOffset and (event.modifierFlags() & NSShiftKeyMask)
        ):
            self.offset = newOffset
            # update observed controller if set
            if self.observedObjectForOffset is not None:
                self.observedObjectForOffset.setValue_forKeyPath_(
                    newOffset, self.observedKeyPathForOffset
                )

        # if we have a multiple selection for angle and Shift key is pressed
        # then don't update the angle
        # this allows angles to remain constant, but change offset
        if not (
            self.multipleSelectionForAngle and (event.modifierFlags() & NSShiftKeyMask)
        ):
            newAngle = atan2(xOffset, yOffset)
            newAngleDegrees = newAngle / (pi / 180.0)
            if newAngleDegrees < 0:
                newAngleDegrees += 360
            self.angle = newAngleDegrees
            # update observed controller if set
            if self.observedObjectForAngle is not None:
                if self.observedObjectForAngle is not None:
                    vt = NSValueTransformer.valueTransformerForName_(
                        self.angleValueTransformerName
                    )
                    newControllerAngle = vt.reverseTransformedValue_(newAngleDegrees)
                else:
                    newControllerAngle = newAngle
            self.observedObjectForAngle.setValue_forKeyPath_(
                newControllerAngle, self.observedKeyPathForAngle
            )
        self.setNeedsDisplay_(True)

    def mouseDown_(self, event):
        self.mouseDown = True
        self.updateForMouseEvent_(event)

    def mouseDragged_(self, event):
        self.updateForMouseEvent_(event)

    def mouseUp_(self, event):
        self.mouseDown = False
        self.updateForMouseEvent_(event)

    def acceptsFirstMouse_(self, event):
        return True

    def acceptsFirstResponder(self):
        return True

    def drawRect_(self, rect):
        """
        Basic goals here:
        If either the angle or the offset has a "bad selection":
        then draw a gray rectangle, and that's it.
        Note: bad selection is set if there's a multiple selection
        but the "allows multiple selection" binding is NO.

        If there's a multiple selection for either angle or offset:
        then what you draw depends on what's multiple.

        - First, draw a white background to show all's OK.

        - If both are multiple, then draw a special symbol.

        - If offset is multiple, draw a line from the center of the view
        - to the edge at the shared angle.

        - If angle is multiple, draw a circle of radius the shared offset
        - centered in the view.

        If neither is multiple, draw a cross at the center of the view
        and a cross at distance 'offset' from the center at angle 'angle'
        """
        myBounds = self.bounds()
        if self.badSelectionForAngle or self.badSelectionForOffset:
            # "disable" and exit
            NSDrawDarkBezel(myBounds, myBounds)
            return
        # user can do something, so draw white background and
        # clip in anticipation of future drawing
        NSDrawLightBezel(myBounds, myBounds)
        clipRect = NSBezierPath.bezierPathWithRect_(NSInsetRect(myBounds, 2.0, 2.0))
        clipRect.addClip()

        if self.multipleSelectionForAngle or self.multipleSelectionForOffset:
            originOffsetX = myBounds.size.width / 2 + 0.5
            originOffsetY = myBounds.size.height / 2 + 0.5
            if self.multipleSelectionForAngle and self.multipleSelectionForOffset:
                # draw a diagonal line and circle to denote
                # multiple selections for angle and offset
                NSBezierPath.strokeLineFromPoint_toPoint_(
                    NSMakePoint(0, 0),
                    NSMakePoint(myBounds.size.width, myBounds.size.height),
                )
                circleBounds = NSMakeRect(originOffsetX - 5, originOffsetY - 5, 10, 10)
                path = NSBezierPath.bezierPathWithOvalInRect_(circleBounds)
                path.stroke()
                return
            if self.multipleSelectionForOffset:
                # draw a line from center to a point outside
                # bounds in the direction specified by angle
                angleRadians = self.angle * (pi / 180.0)
                x = sin(angleRadians) * myBounds.size.width + originOffsetX
                y = cos(angleRadians) * myBounds.size.height + originOffsetX
                NSBezierPath.strokeLineFromPoint_toPoint_(
                    NSMakePoint(originOffsetX, originOffsetY), NSMakePoint(x, y)
                )
                return
            if self.multipleSelectionForAngle:
                # draw a circle with radius the shared offset
                # don't draw radius < 1.0, else invisible
                drawRadius = self.offset
                if drawRadius < 1.0:
                    drawRadius = 1.0
                offsetBounds = NSMakeRect(
                    originOffsetX - drawRadius,
                    originOffsetY - drawRadius,
                    drawRadius * 2,
                    drawRadius * 2,
                )
                path = NSBezierPath.bezierPathWithOvalInRect_(offsetBounds)
                path.stroke()
                return
            # shouldn't get here
            return
        trans = NSAffineTransform.transform()
        trans.translateXBy_yBy_(
            myBounds.size.width / 2 + 0.5, myBounds.size.height / 2 + 0.5
        )
        trans.concat()
        path = NSBezierPath.bezierPath()

        # draw + where shadow extends
        angleRadians = self.angle * (pi / 180.0)
        xOffset = sin(angleRadians) * self.offset
        yOffset = cos(angleRadians) * self.offset

        path.moveToPoint_(NSMakePoint(xOffset, yOffset - 5))
        path.lineToPoint_(NSMakePoint(xOffset, yOffset + 5))
        path.moveToPoint_(NSMakePoint(xOffset - 5, yOffset))
        path.lineToPoint_(NSMakePoint(xOffset + 5, yOffset))

        NSColor.lightGrayColor().set()
        path.setLineWidth_(1.5)
        path.stroke()

        # draw + in center of view
        path = NSBezierPath.bezierPath()

        path.moveToPoint_(NSMakePoint(0, -5))
        path.lineToPoint_(NSMakePoint(0, +5))
        path.moveToPoint_(NSMakePoint(-5, 0))
        path.lineToPoint_(NSMakePoint(+5, 0))

        NSColor.blackColor().set()
        path.setLineWidth_(1.0)
        path.stroke()

    def setNilValueForKey_(self, key):
        """We may get passed nil for angle or offset. Just use 0"""
        self.setValue_forKey_(0, key)

    def validateMaxOffset_error_(self, ioValue, error):
        if ioValue is None:
            # trap this in setNilValueForKey
            # alternative might be to create new NSNumber with value 0 here
            return True
        if ioValue <= 0.0:
            errorString = NSLocalizedStringFromTable(
                "Maximum Offset must be greater than zero",
                "Joystick",
                "validation: zero maxOffset error",
            )
            userInfoDict = {NSLocalizedDescriptionKey: errorString}
            error = NSError.alloc().initWithDomain_code_userInfo_(
                "JoystickView", 1, userInfoDict
            )
            return False, error
        return True, None


JoystickView.exposeBinding_("offset")
JoystickView.exposeBinding_("angle")



import array
import sys

import objc
import AppKit
from objc import NO, YES
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSBitmapImageRep(TestCase):
    def testInstantiation(self):
        # widthxheight RGB 24bpp image
        width = 256
        height = 256
        dataPlanes = (None, None, None, None, None)
        dataPlanes = None
        i1 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            dataPlanes, width, height, 8, 3, NO, NO, AppKit.NSDeviceRGBColorSpace, 0, 0
        )
        self.assertTrue(i1)

        i2 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            None, width, height, 8, 3, NO, NO, AppKit.NSDeviceRGBColorSpace, 0, 0
        )
        self.assertTrue(i2)

    def testPixelFormat(self):
        width = 16
        height = 16

        i1 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bitmapFormat_bytesPerRow_bitsPerPixel_(  # noqa: B950
            None,
            width,
            height,
            8,
            3,
            NO,
            NO,
            AppKit.NSDeviceRGBColorSpace,
            AppKit.NSAlphaFirstBitmapFormat,
            0,
            0,
        )
        self.assertIsInstance(i1, AppKit.NSBitmapImageRep)

        singlePlane = objc.allocateBuffer(width * height * 4)
        for i in range(0, width * height):
            si = i * 4
            singlePlane[si] = 1
            singlePlane[si + 1] = 2
            singlePlane[si + 2] = 3
            singlePlane[si + 3] = 4
        dataPlanes = (singlePlane, None, None, None, None)
        # test non-planar, premade buffer
        i2 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bitmapFormat_bytesPerRow_bitsPerPixel_(  # noqa: B950
            dataPlanes,
            width,
            height,
            8,
            3,
            NO,
            NO,
            AppKit.NSDeviceRGBColorSpace,
            AppKit.NSAlphaFirstBitmapFormat,
            0,
            0,
        )
        self.assertIsInstance(i2, AppKit.NSBitmapImageRep)

        bitmapData = i2.bitmapData()

        self.assertEqual(len(bitmapData), width * height * 4)

    def testImageData(self):
        width = 256
        height = 256

        rPlane = array.array("B")
        rPlane.fromlist([y % 256 for y in range(0, height) for x in range(0, width)])
        if sys.version_info[0] == 3:
            buffer = memoryview
        else:
            from __builtin__ import buffer
        rPlane = buffer(rPlane)

        gPlane = array.array("B")
        gPlane.fromlist(
            [y % 256 for y in range(0, height) for x in range(width, 0, -1)]
        )
        gPlane = buffer(gPlane)

        bPlane = array.array("B")
        bPlane.fromlist([x % 256 for y in range(0, height) for x in range(0, width)])
        bPlane = buffer(bPlane)

        dataPlanes = (rPlane, gPlane, bPlane, None, None)

        # test planar, pre-made buffer
        i1 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            dataPlanes, width, height, 8, 3, NO, YES, AppKit.NSDeviceRGBColorSpace, 0, 0
        )
        self.assertTrue(i1)

        singlePlane = objc.allocateBuffer(width * height * 3)
        for i in range(0, width * height):
            si = i * 3
            if sys.version_info[0] == 2:
                singlePlane[si] = rPlane[i]
                singlePlane[si + 1] = gPlane[i]
                singlePlane[si + 2] = bPlane[i]
            else:

                def as_byte(v):
                    if isinstance(v, int):
                        return v
                    else:
                        return ord(v)

                singlePlane[si] = as_byte(rPlane[i])
                singlePlane[si + 1] = as_byte(gPlane[i])
                singlePlane[si + 2] = as_byte(bPlane[i])

        dataPlanes = (singlePlane, None, None, None, None)
        # test non-planar, premade buffer
        i2 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            dataPlanes, width, height, 8, 3, NO, NO, AppKit.NSDeviceRGBColorSpace, 0, 0
        )

        # test grey scale
        greyPlane = array.array("B")
        greyPlane.fromlist([x % 256 for x in range(0, height) for x in range(0, width)])
        greyPlanes = (greyPlane, None, None, None, None)
        greyImage = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            greyPlanes,
            width,
            height,
            8,
            1,
            NO,
            YES,
            AppKit.NSCalibratedWhiteColorSpace,
            width,
            8,
        )
        self.assertIsNot(greyImage, None)

        # test planar, AppKit.NSBIR allocated buffer
        i3 = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
            None, width, height, 8, 3, NO, YES, AppKit.NSDeviceRGBColorSpace, 0, 0
        )

        r, g, b, a, o = i3.getBitmapDataPlanes_()
        self.assertTrue(r)
        self.assertTrue(g)
        self.assertTrue(b)
        self.assertTrue(not a)
        self.assertTrue(not o)

        self.assertEqual(len(r), len(rPlane))
        self.assertEqual(len(g), len(gPlane))
        self.assertEqual(len(b), len(bPlane))

        r[0 : len(r)] = rPlane[0 : len(rPlane)]
        g[0 : len(g)] = gPlane[0 : len(gPlane)]
        b[0 : len(b)] = bPlane[0 : len(bPlane)]

        bitmapData = i2.bitmapData()

        self.assertEqual(len(bitmapData), len(singlePlane))
        self.assertEqual(bitmapData.tobytes(), singlePlane)

        a = array.array("L", [255] * 4)
        self.assertArgIsOut(AppKit.NSBitmapImageRep.getPixel_atX_y_, 0)
        d = i2.getPixel_atX_y_(a, 1, 1)
        self.assertIs(a, d)


class TestBadCreation(TestCase):

    # Redirect stderr to /dev/null for the duration of this test,
    # AppKit.NSBitmapImageRep will write an error message to stderr.

    def setUp(self):
        import os

        self.duppedStderr = os.dup(2)
        fp = os.open("/dev/null", os.O_RDWR)
        os.dup2(fp, 2)
        os.close(fp)

    def tearDown(self):
        import os

        os.dup2(self.duppedStderr, 2)

    def test_AllocInit(self):
        y = AppKit.NSBitmapImageRep.alloc()
        try:
            self.assertRaises(ValueError, y.init)
        finally:
            width = 256
            height = 256
            dataPlanes = (None, None, None, None, None)
            y = y.initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bytesPerRow_bitsPerPixel_(  # noqa: B950
                dataPlanes,
                width,
                height,
                8,
                3,
                NO,
                NO,
                AppKit.NSDeviceRGBColorSpace,
                0,
                0,
            )

    def testConstants(self):
        self.assertEqual(AppKit.NSTIFFCompressionNone, 1)
        self.assertEqual(AppKit.NSTIFFCompressionCCITTFAX3, 3)
        self.assertEqual(AppKit.NSTIFFCompressionCCITTFAX4, 4)
        self.assertEqual(AppKit.NSTIFFCompressionLZW, 5)
        self.assertEqual(AppKit.NSTIFFCompressionJPEG, 6)
        self.assertEqual(AppKit.NSTIFFCompressionNEXT, 32766)
        self.assertEqual(AppKit.NSTIFFCompressionPackBits, 32773)
        self.assertEqual(AppKit.NSTIFFCompressionOldJPEG, 32865)

        self.assertEqual(AppKit.NSTIFFFileType, 0)
        self.assertEqual(AppKit.NSBMPFileType, 1)
        self.assertEqual(AppKit.NSGIFFileType, 2)
        self.assertEqual(AppKit.NSJPEGFileType, 3)
        self.assertEqual(AppKit.NSPNGFileType, 4)
        self.assertEqual(AppKit.NSJPEG2000FileType, 5)

        self.assertEqual(AppKit.NSBitmapImageFileTypeTIFF, 0)
        self.assertEqual(AppKit.NSBitmapImageFileTypeBMP, 1)
        self.assertEqual(AppKit.NSBitmapImageFileTypeGIF, 2)
        self.assertEqual(AppKit.NSBitmapImageFileTypeJPEG, 3)
        self.assertEqual(AppKit.NSBitmapImageFileTypePNG, 4)
        self.assertEqual(AppKit.NSBitmapImageFileTypeJPEG2000, 5)

        self.assertEqual(AppKit.NSBitmapFormatAlphaFirst, 1 << 0)
        self.assertEqual(AppKit.NSBitmapFormatAlphaNonpremultiplied, 1 << 1)
        self.assertEqual(AppKit.NSBitmapFormatFloatingPointSamples, 1 << 2)
        self.assertEqual(AppKit.NSBitmapFormatSixteenBitLittleEndian, 1 << 8)
        self.assertEqual(AppKit.NSBitmapFormatThirtyTwoBitLittleEndian, 1 << 9)
        self.assertEqual(AppKit.NSBitmapFormatSixteenBitBigEndian, 1 << 10)
        self.assertEqual(AppKit.NSBitmapFormatThirtyTwoBitBigEndian, 1 << 11)

        self.assertEqual(AppKit.NSImageRepLoadStatusUnknownType, -1)
        self.assertEqual(AppKit.NSImageRepLoadStatusReadingHeader, -2)
        self.assertEqual(AppKit.NSImageRepLoadStatusWillNeedAllData, -3)
        self.assertEqual(AppKit.NSImageRepLoadStatusInvalidData, -4)
        self.assertEqual(AppKit.NSImageRepLoadStatusUnexpectedEOF, -5)
        self.assertEqual(AppKit.NSImageRepLoadStatusCompleted, -6)

        self.assertEqual(AppKit.NSAlphaFirstBitmapFormat, 1 << 0)
        self.assertEqual(AppKit.NSAlphaNonpremultipliedBitmapFormat, 1 << 1)
        self.assertEqual(AppKit.NSFloatingPointSamplesBitmapFormat, 1 << 2)

        self.assertIsInstance(AppKit.NSImageCompressionMethod, str)
        self.assertIsInstance(AppKit.NSImageCompressionFactor, str)
        self.assertIsInstance(AppKit.NSImageDitherTransparency, str)
        self.assertIsInstance(AppKit.NSImageRGBColorTable, str)
        self.assertIsInstance(AppKit.NSImageInterlaced, str)
        self.assertIsInstance(AppKit.NSImageColorSyncProfileData, str)
        self.assertIsInstance(AppKit.NSImageFrameCount, str)
        self.assertIsInstance(AppKit.NSImageCurrentFrame, str)
        self.assertIsInstance(AppKit.NSImageCurrentFrameDuration, str)
        self.assertIsInstance(AppKit.NSImageLoopCount, str)
        self.assertIsInstance(AppKit.NSImageGamma, str)
        self.assertIsInstance(AppKit.NSImageProgressive, str)
        self.assertIsInstance(AppKit.NSImageEXIFData, str)
        self.assertIsInstance(AppKit.NSImageFallbackBackgroundColor, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NS16BitLittleEndianBitmapFormat, (1 << 8))
        self.assertEqual(AppKit.NS32BitLittleEndianBitmapFormat, (1 << 9))
        self.assertEqual(AppKit.NS16BitBigEndianBitmapFormat, (1 << 10))
        self.assertEqual(AppKit.NS32BitBigEndianBitmapFormat, (1 << 11))

    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertIsInstance(AppKit.NSImageIPTCData, str)

    def testTiffCompression(self):
        lst, nr = AppKit.NSBitmapImageRep.getTIFFCompressionTypes_count_(None, None)
        self.assertIsInstance(lst, tuple)
        self.assertIsInstance(nr, int)
        self.assertEqual(len(lst), nr)
        self.assertNotEqual(len(lst), 0)
        self.assertIsInstance(lst[0], int)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBitmapImageRep.isPlanar)
        self.assertResultIsBOOL(AppKit.NSBitmapImageRep.canBeCompressedUsing_)
        self.assertArgIsBOOL(
            AppKit.NSBitmapImageRep.incrementalLoadFromData_complete_, 1
        )

        self.assertArgIsOut(AppKit.NSBitmapImageRep.getCompression_factor_, 0)
        self.assertArgIsOut(AppKit.NSBitmapImageRep.getCompression_factor_, 1)



#
#  FontSampleDisplayView.py
#  ControlledPreferences
#
#  Converted by u.fiedler on 04.02.05.
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Cocoa import (
    NSAttributedString,
    NSDrawLightBezel,
    NSFont,
    NSFontAttributeName,
    NSInsetRect,
    NSMakePoint,
    NSNotificationCenter,
    NSRectFill,
    NSUnarchiver,
    NSUserDefaultsController,
    NSUserDefaultsDidChangeNotification,
    NSView,
)
from objc import super
from PyObjCTools.KeyValueCoding import getKey


class FontSampleDisplayView(NSView):
    """
    Display WordOfTheDay with the preferred font and color
    """

    def drawRect_(self, rect):
        defaults = NSUserDefaultsController.sharedUserDefaultsController().values()
        favoriteColor = NSUnarchiver.unarchiveObjectWithData_(
            getKey(defaults, "FavoriteColor")
        )
        fontName = getKey(defaults, "FontName")
        fontSize = getKey(defaults, "FontSize")
        favoriteFont = NSFont.fontWithName_size_(fontName, fontSize)
        wordOfTheDay = getKey(defaults, "WordOfTheDay")

        # Do the actual drawing
        myBounds = self.bounds()  # = (x, y), (bw, bh)
        NSDrawLightBezel(myBounds, myBounds)
        favoriteColor.set()
        NSRectFill(NSInsetRect(myBounds, 2, 2))
        attrsDictionary = {NSFontAttributeName: favoriteFont}
        attrString = NSAttributedString.alloc().initWithString_attributes_(
            wordOfTheDay, attrsDictionary
        )
        attrSize = attrString.size()  # = (bw, bh)
        attrString.drawAtPoint_(
            NSMakePoint(
                (attrSize.width / -2) + (myBounds.size.width / 2),
                (attrSize.height / -2) + (myBounds.size.height / 2),
            )
        )

    def initWithFrame_(self, frameRect):
        self = super().initWithFrame_(frameRect)
        dnc = NSNotificationCenter.defaultCenter()
        dnc.addObserver_selector_name_object_(
            self, "redisplay:", NSUserDefaultsDidChangeNotification, None
        )
        return self

    def redisplay_(self, sender):
        self.setNeedsDisplay_(True)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSObjectHelper(Foundation.NSObject):
    def copyWithZone_(self, zn):
        return None

    def mutableCopyWithZone_(self, zn):
        return None

    def isContentDiscarded(self):
        return 1

    def beginContentAccess(self):
        return 1

    @classmethod
    def supportsSecureCoding(self):
        return 1

    def retainWeakReference(self):
        return 0

    def allowsWeakReference(self):
        return 0


class TestNSObjectFunctions(TestCase):
    def testAllocation(self):
        o = Foundation.NSAllocateObject(Foundation.NSObject, 0, None)
        self.assertIsInstance(o, Foundation.NSObject)
        o = Foundation.NSAllocateObject(Foundation.NSObject, 50, None)
        self.assertIsInstance(o, Foundation.NSObject)
        Foundation.NSDeallocateObject(None)

    def testCopy(self):
        o = Foundation.NSObject.alloc().init()
        self.assertIsInstance(o, Foundation.NSObject)
        o2 = Foundation.NSCopyObject(o, 50, None)
        self.assertIsInstance(o2, Foundation.NSObject)

    def testShouldRetain(self):
        o = Foundation.NSObject.alloc().init()
        self.assertIsInstance(o, Foundation.NSObject)
        v = Foundation.NSShouldRetainWithZone(o, None)
        self.assertTrue((v is True) or (v is False))

    def testRefCounts(self):
        o = Foundation.NSObject.alloc().init()

        cnt = Foundation.NSExtraRefCount(o)
        self.assertIsInstance(cnt, int)
        Foundation.NSIncrementExtraRefCount(o)
        v = Foundation.NSExtraRefCount(o)
        self.assertEqual(v, cnt + 1)

        v = Foundation.NSDecrementExtraRefCountWasZero(o)
        self.assertIs(v, False)
        v = Foundation.NSExtraRefCount(o)
        self.assertEqual(v, cnt)

    @min_os_level("10.7")
    def testFunctions10_7(self):
        # No further testing needed:
        Foundation.CFBridgingRetain
        Foundation.CFBridgingRelease


class TestNSObjectInteraction(TestCase):
    def testCallingInstanceMethodWithClassSelf(self):
        self.assertRaises(
            TypeError, Foundation.NSObject.description, Foundation.NSObject
        )
        self.assertRaises(TypeError, Foundation.NSObject.description, "hello")

    def testNSObjectClassMethod(self):
        # Check that -class is accesible as 'class__' and 'class' (the latter
        # only through getattr because it is a Python keyword)
        self.assertTrue(hasattr(Foundation.NSObject, "class__"))
        self.assertTrue(isinstance(Foundation.NSObject.class__, objc.selector))
        o = Foundation.NSObject.alloc().init()
        self.assertTrue(o.class__() is o.__class__)

        self.assertTrue(hasattr(Foundation.NSObject, "class"))
        self.assertTrue(
            isinstance(getattr(Foundation.NSObject, "class"), objc.selector)
        )
        self.assertTrue(getattr(o, "class")() is o.__class__)

    def testNSObjectClass(self):
        self.assertTrue(
            Foundation.NSObject.instancesRespondToSelector_("description"),
            "NSObject class claims it doesn't respond to a selector that it does.",
        )
        self.assertTrue(
            hasattr(Foundation.NSObject, "description"),
            "NSObject class claims it doesn't respond to a selector that it does.",
        )
        self.assertTrue(
            Foundation.NSObject.description(),
            "NSObject class failed to respond to +description selector.",
        )

    def testNSObjectInstance(self):
        instance = Foundation.NSObject.new()

        self.assertTrue(instance, "Failed to instantiate an instance")
        self.assertTrue(
            instance.description(),
            "NSObject instance didn't respond to -description selector.",
        )
        self.assertTrue(
            not instance.isProxy(),
            "Instance of Foundation.NSObject claimed it was a proxy.   That seems odd.",
        )
        self.assertTrue(
            isinstance(instance, Foundation.NSObject),
            "Instantiated object not an instance of Foundation.NSObject.",
        )
        self.assertEqual(instance, instance, "Python identity check failed.")
        self.assertTrue(instance.isEqual_(instance), "Obj-C identity check failed.")

    def testRepeatedAllocInit(self):
        for _ in range(1, 1000):
            _ = Foundation.NSObject.alloc().init()

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSObject.isEqual_)
        self.assertResultIsBOOL(Foundation.NSObject.isProxy)
        self.assertResultIsBOOL(Foundation.NSObject.isKindOfClass_)
        self.assertResultIsBOOL(Foundation.NSObject.isMemberOfClass_)
        self.assertResultIsBOOL(Foundation.NSObject.conformsToProtocol_)
        self.assertResultIsBOOL(Foundation.NSObject.respondsToSelector_)
        self.assertResultIsBOOL(Foundation.NSObject.instancesRespondToSelector_)
        self.assertResultIsBOOL(Foundation.NSObject.isSubclassOfClass_)
        self.assertResultIsBOOL(Foundation.NSObject.resolveClassMethod_)
        self.assertResultIsBOOL(Foundation.NSObject.resolveInstanceMethod_)

        o = Foundation.NSObject.alloc().init()
        self.assertResultIsBOOL(o.isEqual_)
        self.assertResultIsBOOL(o.isProxy)
        self.assertResultIsBOOL(o.isKindOfClass_)
        self.assertResultIsBOOL(o.isMemberOfClass_)
        self.assertResultIsBOOL(o.conformsToProtocol_)
        self.assertResultIsBOOL(o.respondsToSelector_)

        a = TestNSObjectHelper.alloc().init()
        self.assertArgHasType(a.copyWithZone_, 0, b"^{_NSZone=}")
        self.assertArgHasType(a.mutableCopyWithZone_, 0, b"^{_NSZone=}")

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(TestNSObjectHelper.beginContentAccess)
        self.assertResultIsBOOL(TestNSObjectHelper.isContentDiscarded)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(TestNSObjectHelper.allowsWeakReference)
        self.assertResultIsBOOL(TestNSObjectHelper.retainWeakReference)

        self.assertResultIsBOOL(TestNSObjectHelper.supportsSecureCoding)

    def testProtocols(self):
        objc.protocolNamed("NSCopying")
        objc.protocolNamed("NSMutableCopying")
        objc.protocolNamed("NSCoding")

    @min_sdk_level("10.6")
    def testProtocols10_6(self):
        objc.protocolNamed("NSDiscardableContent")

    @min_sdk_level("10.7")
    def testProtocols10_7(self):
        objc.protocolNamed("NSSecureCoding")



import CoreFoundation
from Foundation import NSArray, NSMutableArray
from PyObjCTools.TestSupport import TestCase


def cmp(a, b):
    if a < b:
        return -1
    elif b < a:
        return 1
    return 0


class TestCFArray(TestCase):
    def testCFArrayIsNSArray(self):
        self.assertTrue(issubclass(CoreFoundation.CFArrayRef, NSArray))
        self.assertTrue(issubclass(CoreFoundation.CFMutableArrayRef, NSMutableArray))

    def testCFArrayCreate(self):
        array = CoreFoundation.CFArrayCreate(
            None, [1, 2, 3, 4], 4, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, [1, 2, 3, 4])
        self.assertIsInstance(array, NSArray)

        array = CoreFoundation.CFArrayCreateMutable(
            None, 0, CoreFoundation.kCFTypeArrayCallBacks
        )
        CoreFoundation.CFArrayAppendValue(array, 42)
        CoreFoundation.CFArrayAppendValue(array, 43)
        CoreFoundation.CFArrayAppendValue(array, 44)
        self.assertEqual(array, [42, 43, 44])
        self.assertIsInstance(array, CoreFoundation.CFMutableArrayRef)

    def testCFArrayApplyFunction(self):
        array = CoreFoundation.CFArrayCreate(
            None, [1, 2, 3, 4], 4, CoreFoundation.kCFTypeArrayCallBacks
        )

        self.assertArgIsFunction(CoreFoundation.CFArrayApplyFunction, 2, b"v@@", False)
        self.assertArgHasType(CoreFoundation.CFArrayApplyFunction, 3, b"@")

        items = []
        infos = []

        def applier(item, info):
            items.append(item * item)
            infos.append(info)

        CoreFoundation.CFArrayApplyFunction(array, (0, 4), applier, 42)
        self.assertEqual(items, [1, 4, 9, 16])
        self.assertEqual(infos, [42, 42, 42, 42])

        items = []
        infos = []
        CoreFoundation.CFArrayApplyFunction(array, (1, 2), applier, 42)
        self.assertEqual(items, [4, 9])
        self.assertEqual(infos, [42, 42])

    def testBSearchValues(self):
        # This method causes a hard crash, reason unclear.
        array = CoreFoundation.CFArrayCreate(
            None, range(20), 20, CoreFoundation.kCFTypeArrayCallBacks
        )

        self.assertArgHasType(CoreFoundation.CFArrayBSearchValues, 2, b"@")
        self.assertArgIsFunction(CoreFoundation.CFArrayBSearchValues, 3, b"l@@@", False)
        self.assertArgHasType(CoreFoundation.CFArrayBSearchValues, 4, b"@")

        def compare(a, b, context):
            return cmp(a, b)

        r = CoreFoundation.CFArrayBSearchValues(array, (0, 20), 10, compare, None)
        self.assertEqual(r, 10)

        r = CoreFoundation.CFArrayBSearchValues(array, (0, 20), 9.5, compare, None)
        self.assertEqual(r, 10)

        r = CoreFoundation.CFArrayBSearchValues(array, (0, 20), 99, compare, None)
        self.assertTrue(r >= 20)

        r = CoreFoundation.CFArrayBSearchValues(array, (0, 20), -1, compare, None)
        self.assertEqual(r, 0)

    def testSortValues(self):
        array = CoreFoundation.NSMutableArray.arrayWithArray_([4, 2, 1, 3, 0, 5])

        self.assertArgIsFunction(CoreFoundation.CFArraySortValues, 2, b"l@@@", False)
        self.assertArgHasType(CoreFoundation.CFArraySortValues, 3, b"@")

        def compare(a, b, context):
            return cmp(a, b)

        CoreFoundation.CFArraySortValues(array, (0, 6), compare, None)

        self.assertEqual(array, [0, 1, 2, 3, 4, 5])

    def testTypeID(self):
        v = CoreFoundation.CFArrayGetTypeID()
        self.assertIsInstance(v, int)

    def testCopy(self):
        array = CoreFoundation.CFArrayCreate(
            None, [1, 2, 3, 4], 4, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, [1, 2, 3, 4])
        self.assertIsInstance(array, NSArray)

        cpy = CoreFoundation.CFArrayCreateCopy(None, array)
        self.assertEqual(cpy, [1, 2, 3, 4])
        self.assertIsInstance(cpy, NSArray)

        cpy = CoreFoundation.CFArrayCreateMutableCopy(None, 0, array)
        self.assertEqual(cpy, [1, 2, 3, 4])
        self.assertIsInstance(cpy, CoreFoundation.CFMutableArrayRef)
        self.assertIsNot(cpy, array)

    def testCounts(self):
        array = CoreFoundation.CFArrayCreate(
            None, [1, 2, 3, 4, 4, 2], 6, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, [1, 2, 3, 4, 4, 2])
        self.assertIsInstance(array, NSArray)

        self.assertEqual(CoreFoundation.CFArrayGetCount(array), 6)
        self.assertEqual(CoreFoundation.CFArrayGetCountOfValue(array, (0, 6), 4), 2)
        self.assertEqual(CoreFoundation.CFArrayGetCountOfValue(array, (0, 6), 2), 2)
        self.assertEqual(CoreFoundation.CFArrayGetCountOfValue(array, (0, 6), 3), 1)

    def testContains(self):
        array = CoreFoundation.CFArrayCreate(
            None, ["a", 2, 3, 4, 4, 2], 6, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, ["a", 2, 3, 4, 4, 2])
        self.assertIsInstance(array, NSArray)

        self.assertFalse(CoreFoundation.CFArrayContainsValue(array, (0, 6), "hello"))
        self.assertTrue(CoreFoundation.CFArrayContainsValue(array, (0, 6), 4))
        self.assertFalse(CoreFoundation.CFArrayContainsValue(array, (0, 2), 4))

        self.assertEqual(
            CoreFoundation.CFArrayGetFirstIndexOfValue(array, (0, 6), 3), 2
        )
        self.assertEqual(
            CoreFoundation.CFArrayGetFirstIndexOfValue(array, (0, 6), 2), 1
        )
        self.assertEqual(
            CoreFoundation.CFArrayGetFirstIndexOfValue(array, (0, 6), "hello"),
            CoreFoundation.kCFNotFound,
        )
        self.assertEqual(CoreFoundation.CFArrayGetLastIndexOfValue(array, (0, 6), 3), 2)
        self.assertEqual(CoreFoundation.CFArrayGetLastIndexOfValue(array, (0, 6), 2), 5)
        self.assertEqual(
            CoreFoundation.CFArrayGetLastIndexOfValue(array, (0, 6), "hello"),
            CoreFoundation.kCFNotFound,
        )
        self.assertArgHasType(CoreFoundation.CFArrayGetFirstIndexOfValue, 2, b"@")
        self.assertArgHasType(CoreFoundation.CFArrayGetLastIndexOfValue, 2, b"@")

    def testGetting(self):
        array = CoreFoundation.CFArrayCreate(
            None, ["a", 2, 3, 4, 4, 2], 6, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, ["a", 2, 3, 4, 4, 2])
        self.assertIsInstance(array, NSArray)

        self.assertEqual(CoreFoundation.CFArrayGetValueAtIndex(array, 0), "a")
        self.assertEqual(CoreFoundation.CFArrayGetValueAtIndex(array, 1), 2)
        self.assertArgHasType(CoreFoundation.CFArrayGetValues, 2, b"o^@")
        self.assertArgSizeInArg(CoreFoundation.CFArrayGetValues, 2, 1)

        vals = CoreFoundation.CFArrayGetValues(array, (0, 3), None)
        self.assertIsInstance(vals, tuple)
        self.assertEqual(vals, ("a", 2, 3))

    def testUpdating(self):
        array = CoreFoundation.CFArrayCreate(
            None, ["a", 2, 3, 4, 4, 2], 6, CoreFoundation.kCFTypeArrayCallBacks
        )
        self.assertEqual(array, ["a", 2, 3, 4, 4, 2])
        self.assertIsInstance(array, CoreFoundation.NSArray)
        array = CoreFoundation.CFArrayCreateMutableCopy(None, 0, array)

        self.assertArgHasType(CoreFoundation.CFArrayAppendValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFArrayInsertValueAtIndex, 2, b"@")
        self.assertArgHasType(CoreFoundation.CFArraySetValueAtIndex, 2, b"@")

        CoreFoundation.CFArrayAppendValue(array, "foo")
        self.assertEqual(array, ["a", 2, 3, 4, 4, 2, "foo"])

        CoreFoundation.CFArrayInsertValueAtIndex(array, 1, 4)
        self.assertEqual(array, ["a", 4, 2, 3, 4, 4, 2, "foo"])

        CoreFoundation.CFArrayRemoveValueAtIndex(array, 2)
        self.assertEqual(array, ["a", 4, 3, 4, 4, 2, "foo"])

        CoreFoundation.CFArraySetValueAtIndex(array, 2, "two")
        self.assertEqual(array, ["a", 4, "two", 4, 4, 2, "foo"])

        CoreFoundation.CFArrayExchangeValuesAtIndices(array, 1, 2)
        self.assertEqual(array, ["a", "two", 4, 4, 4, 2, "foo"])

        self.assertArgHasType(CoreFoundation.CFArrayReplaceValues, 2, b"n^@")
        self.assertArgSizeInArg(CoreFoundation.CFArrayReplaceValues, 2, 3)
        CoreFoundation.CFArrayReplaceValues(
            array, (2, 3), ("a", "b", "c", "d", "e", "f"), 6
        )
        self.assertEqual(array, ["a", "two", "a", "b", "c", "d", "e", "f", 2, "foo"])

        array2 = CoreFoundation.CFArrayCreate(
            None, ["hello", "earth"], 2, CoreFoundation.kCFTypeArrayCallBacks
        )
        CoreFoundation.CFArrayAppendArray(array, array2, (0, 2))
        self.assertEqual(
            array,
            ["a", "two", "a", "b", "c", "d", "e", "f", 2, "foo", "hello", "earth"],
        )

        CoreFoundation.CFArrayRemoveAllValues(array)
        self.assertEqual(array, [])



import sys

import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

if sys.version_info[0] == 3:

    def cmp(a, b):
        if a < b:
            return -1
        elif b < a:
            return 1
        return 0


class TestNSArrayInteraction(TestCase):
    def testRepeatedAllocInit(self):
        for _ in range(1, 1000):
            _ = Foundation.NSArray.alloc().init()

    def testIndices(self):
        x = Foundation.NSArray.arrayWithArray_(["foo", "bar", "baz"])

        self.assertEqual(x.indexOfObject_("bar"), 1)

        self.assertRaises(IndexError, x.objectAtIndex_, 100)

    def testEnumeration(self):
        x = Foundation.NSArray.arrayWithArray_([1, 2, "foo", "bar", "", "baz"])
        y = []

        for o in x:
            y.append(o)

        self.assertEqual(len(x), len(y))

    def testContains(self):
        x = Foundation.NSArray.arrayWithArray_(["foo", "bar", "baz"])
        self.assertEqual(x.count(), 3)
        self.assertEqual(len(x), 3)

        self.assertTrue(x.containsObject_("foo"))
        self.assertTrue(not x.containsObject_("dumbledorf"))

        self.assertTrue("foo" in x)
        self.assertTrue("dumbledorf" not in x)

    def testIn(self):
        x = Foundation.NSMutableArray.array()
        for i in range(0, 100):
            x.addObject_(i)

        y = []
        for i in x:
            y.append(i)

        z = []
        for i in range(0, 100):
            z.append(i)

        self.assertEqual(x, y)
        self.assertEqual(x, z)
        self.assertEqual(y, z)

        for i in range(0, 100):
            self.assertTrue(i in x)

        self.assertTrue(101 not in x)
        self.assertTrue(None not in x)
        self.assertTrue("foo bar" not in x)

    def assertSlicesEqual(self, x, y, z):
        self.assertEqual(x, x[:])
        self.assertEqual(y, y[:])
        self.assertEqual(z, z[:])

        self.assertEqual(x[25:75], y[25:75])
        self.assertEqual(x[25:75], z[25:75])
        self.assertEqual(y[25:75], z[25:75])

        self.assertEqual(x[:15], y[:15])
        self.assertEqual(x[:15], z[:15])
        self.assertEqual(y[:15], z[:15])

        self.assertEqual(x[15:], y[15:])
        self.assertEqual(x[15:], z[15:])
        self.assertEqual(y[15:], z[15:])

        self.assertEqual(x[-15:], y[-15:])
        self.assertEqual(x[-15:], z[-15:])
        self.assertEqual(y[-15:], z[-15:])

        self.assertEqual(x[-15:30], y[-15:30])
        self.assertEqual(x[-15:30], z[-15:30])
        self.assertEqual(y[-15:30], z[-15:30])

        self.assertEqual(x[-15:-5], y[-15:-5])
        self.assertEqual(x[-15:-5], z[-15:-5])
        self.assertEqual(y[-15:-5], z[-15:-5])

    def testSlice(self):
        x = Foundation.NSMutableArray.array()
        for i in range(0, 100):
            x.addObject_(i)

        y = []
        for i in x:
            y.append(i)

        z = []
        for i in range(0, 100):
            z.append(i)

        self.assertSlicesEqual(x, y, z)

        k = range(300, 50)
        x[20:30] = k
        y[20:30] = k
        z[20:30] = k

        self.assertSlicesEqual(x, y, z)

        # Note that x[1] = x works in python, but not for a bridged Foundation.NS*Array*.
        # Not sure if there is anything we can do about that.
        x[1] = x[:]
        y[1] = y[:]
        z[1] = z[:]

        self.assertSlicesEqual(x, y, z)

        del x[-15:-5]
        del y[-15:-5]
        del z[-15:-5]

        self.assertSlicesEqual(x, y, z)

    def test_mixSliceNDice(self):
        # This test failes on Python 2.2, that is expected.
        x = list(range(0, 10))
        y = Foundation.NSMutableArray.arrayWithArray_(range(0, 10))

        y[2:4] = x[1:5]
        x[2:8] = y[3:7]
        y[2:4] = y[1:8]

    def test_subScripts(self):
        x = list(range(0, 10))
        y = Foundation.NSMutableArray.arrayWithArray_(x)

        self.assertEqual(x[0], y[0])
        self.assertEqual(x[2], y[2])

        self.assertEqual(x[-1], y[-1])
        self.assertEqual(x[-5], y[-5])

        self.assertRaises(IndexError, x.__getitem__, 100)
        self.assertRaises(IndexError, x.__getitem__, -100)

    def test_varargConstruction(self):
        w = Foundation.NSArray.arrayWithObjects_(1, 2, 3, 4)
        x = Foundation.NSArray.alloc().initWithObjects_(1, 2, 3, 4)

        y = Foundation.NSArray.arrayWithObjects_count_([1, 2, 3, 4, 5, 6], 4)
        z = Foundation.NSArray.alloc().initWithObjects_count_([1, 2, 3, 4, 5, 6], 4)

        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)

        self.assertEqual(w[0], 1)
        self.assertEqual(x[1], 2)
        self.assertEqual(y[2], 3)
        self.assertEqual(z[3], 4)

    def test_varargConstruction2(self):
        w = Foundation.NSMutableArray.arrayWithObjects_(1, 2, 3, 4, None)
        x = Foundation.NSMutableArray.alloc().initWithObjects_(1, 2, 3, 4, None)
        y = Foundation.NSMutableArray.arrayWithObjects_count_([1, 2, 3, 4, 5, 6], 4)
        z = Foundation.NSMutableArray.alloc().initWithObjects_count_(
            [1, 2, 3, 4, 5, 6], 4
        )

        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)

        self.assertEqual(w[0], 1)
        self.assertEqual(x[1], 2)
        self.assertEqual(y[2], 3)
        self.assertEqual(z[3], 4)


class TestNSArraySpecialMethods(TestCase):
    # Test calling 'difficult' methods from Python

    def test_initWithObjects_count_(self):
        a = Foundation.NSArray.alloc().initWithObjects_count_(("a", "b", "c", "d"), 3)
        self.assertEqual(a, ["a", "b", "c"])

        import warnings

        warnings.filterwarnings("ignore", category=objc.UninitializedDeallocWarning)

        try:
            self.assertRaises(
                ValueError,
                Foundation.NSArray.alloc().initWithObjects_count_,
                ("a", "b"),
                3,
            )

        finally:
            del warnings.filters[0]

    def test_arrayWithObjects_count_(self):
        a = Foundation.NSArray.arrayWithObjects_count_(("a", "b", "c", "d"), 3)
        self.assertEqual(a, ["a", "b", "c"])

        self.assertRaises(
            ValueError, Foundation.NSArray.arrayWithObjects_count_, ("a", "b"), 3
        )

    def test_arrayByAddingObjects_count_(self):
        return

        a = Foundation.NSArray.arrayWithArray_(("a", "b", "c"))
        self.assertEqual(a, ("a", "b", "c"))

        b = a.arrayByAddingObjects_count_(("d", "e", "f"), 3)
        self.assertEqual(a, ("a", "b", "c"))
        self.assertEqual(b, ("a", "b", "c", "d", "e", "f"))

        self.assertRaises(ValueError, a.arrayByAddingObjects_count_, ("a", "b"), 3)

    def test_sortedArrayUsingFunction_context_(self):
        a = Foundation.NSArray.arrayWithArray_(("a", "b", "c"))
        self.assertEqual(a, ("a", "b", "c"))

        def cmpfunc(a, b, c):
            return -cmp(a, b)

        b = a.sortedArrayUsingFunction_context_(cmpfunc, "hello")
        self.assertEqual(a, ("a", "b", "c"))
        self.assertEqual(b, ("c", "b", "a"))

    def test_sortedArrayUsingFunction_context_hint_(self):
        a = Foundation.NSArray.arrayWithArray_(("a", "b", "c"))
        self.assertEqual(a, ("a", "b", "c"))

        def cmpfunc(a, b, c):
            return -cmp(a, b)

        b = a.sortedArrayUsingFunction_context_hint_(
            cmpfunc, "hello", a.sortedArrayHint()
        )
        self.assertEqual(a, ("a", "b", "c"))
        self.assertEqual(b, ("c", "b", "a"))


class TestNSMutableArrayInteraction(TestCase):
    def testRemoveObjects(self):
        a = Foundation.NSMutableArray.arrayWithArray_(range(10))

        self.assertEqual(len(a), 10)
        self.assertEqual(a[0], 0)
        self.assertEqual(a[1], 1)
        self.assertEqual(a[2], 2)

        a.removeObjectsFromIndices_numIndices_([2, 4, 6, 8], 3)

        self.assertEqual(len(a), 7)
        self.assertEqual(a, (0, 1, 3, 5, 7, 8, 9))

    def testReplaceObjects(self):
        if objc.platform == "MACOSX" or hasattr(
            Foundation.NSMutableArray, "replaceObjectsInRange_withObjects_count_"
        ):

            a = Foundation.NSMutableArray.arrayWithArray_(range(4))
            self.assertEqual(a, (0, 1, 2, 3))

            a.replaceObjectsInRange_withObjects_count_((1, 2), ["a", "b", "c", "d"], 3)

            self.assertEqual(a, (0, "a", "b", "c", 3))

    def testSortInvalid(self):
        # Invalid calls to sortUsingFunction:context:
        def cmp(a, b):
            return -1

        a = Foundation.NSMutableArray.arrayWithArray_(range(4))
        self.assertEqual(a, (0, 1, 2, 3))

        t = objc.options.verbose
        # objc.options.verbose = True
        try:
            self.assertRaises(TypeError, a.sortUsingFunction_context_, dir)
            self.assertRaises(TypeError, a.sortUsingFunction_context_, dir, 1, 2)
            self.assertRaises(
                TypeError, a.sortUsingFunction_context_, lambda *args: cmp(*args), "a"
            )
        finally:
            objc.options.verbose = t

    def dont_testSort2(self):
        # sortUsingFunction:context:range: isn't documented an hence shouldn't be tested
        a = Foundation.NSMutableArray.arrayWithArray_(range(10))
        self.assertEqual(a, (0, 1, 2, 3, 4, 5, 6, 7, 8, 9))

        if objc.platform == "MACOSX" or hasattr(a, "sortUsingFunction_context_range_"):

            def cmpfunc(a, b, c):
                return -cmp(a, b)

            a.sortUsingFunction_context_range_(cmpfunc, "a", (4, 4))

            self.assertEqual(a, (0, 1, 2, 3, 7, 6, 5, 4, 8, 9))

    def testSort1(self):
        a = Foundation.NSMutableArray.arrayWithArray_(range(4))
        self.assertEqual(a, (0, 1, 2, 3))

        def cmpfunc(a, b, c):
            return -cmp(a, b)

        a.sortUsingFunction_context_(cmpfunc, "a")

        self.assertEqual(a, (3, 2, 1, 0))

    def dont_testSort2_2(self):  # XXX
        a = Foundation.NSMutableArray.arrayWithArray_(range(10))
        self.assertEqual(a, (0, 1, 2, 3, 4, 5, 6, 7, 8, 9))

        if objc.platform == "MACOSX" or hasattr(a, "sortUsingFunction_context_range_"):

            def cmpfunc(a, b, c):
                return -cmp(a, b)

            a.sortUsingFunction_context_range_(cmpfunc, "a", (4, 4))

            self.assertEqual(a, (0, 1, 2, 3, 7, 6, 5, 4, 8, 9))

    def testSort4(self):
        # check the sort method, list interface compatibility

        a = Foundation.NSMutableArray.arrayWithArray_(range(4))
        self.assertEqual(a, (0, 1, 2, 3))

        def cmpfunc(a, b):
            return -cmp(a, b)

        if sys.version_info[0] == 2:
            a.sort(cmp=cmpfunc)
            self.assertEqual(a, (3, 2, 1, 0))
        else:
            self.assertRaises(TypeError, a.sort, cmp=cmpfunc)

        a.sort()
        self.assertEqual(a, (0, 1, 2, 3))

        mapping = {0: "nul", 1: "een", 2: "twee", 3: "drie"}

        def keyfunc(item):
            return mapping[item]

        a.sort(key=keyfunc)
        self.assertEqual(a, (3, 1, 0, 2))

        a.sort(key=keyfunc, reverse=True)
        self.assertEqual(a, (2, 0, 1, 3))

        a.sort(reverse=True)
        self.assertEqual(a, (3, 2, 1, 0))

    def getObjectsRange(self):
        o = Foundation.NSArray.arrayWithArray_(range(4, 8))
        v = o.getObjects_range_((1, 2))
        self.assertEqual(v, (5, 6))

    def test_unsupportedMethods(self):
        #
        # Check that calling unsupported methods results in a TypeError
        #
        # NOTE: Some of these don't even exist on GNUstep
        o = Foundation.NSArray.arrayWithArray_(range(4))
        self.assertRaises(TypeError, o.getObjects_)

        #
        # if objc.platform == 'MACOSX' or hasattr(o, 'apply_context_'):
        #    self.assertRaises(TypeError, o.apply_context_, lambda x, y:None, 0)

    def testInsert(self):
        o = Foundation.NSMutableArray.arrayWithArray_(range(4))
        self.assertEqual(list(o), list(range(4)))

        self.assertEqual(o[0], 0)
        o.insert(0, "foo")
        self.assertEqual(o[0], "foo")
        self.assertEqual(o[1], 0)
        self.assertEqual(len(o), 5)

        # FIXME: test the entire interface of list.insert


class TestVariadic(TestCase):
    def testArrayWithObjects(self):
        a = Foundation.NSArray.arrayWithObjects_("foo", "bar", None)
        self.assertEqual(a, ("foo", "bar"))
        self.assertIsInstance(a, Foundation.NSArray)

        a = Foundation.NSMutableArray.arrayWithObjects_("foo", "bar", None)
        self.assertEqual(a, ["foo", "bar"])
        self.assertIsInstance(a, Foundation.NSMutableArray)

    def testInitWithObjecs(self):
        a = Foundation.NSArray.alloc().initWithObjects_("foo", "bar", None)
        self.assertEqual(a, ("foo", "bar"))
        self.assertIsInstance(a, Foundation.NSArray)

        a = Foundation.NSMutableArray.alloc().initWithObjects_("foo", "bar", None)
        self.assertEqual(a, ["foo", "bar"])
        self.assertIsInstance(a, Foundation.NSMutableArray)


class TestNSArray(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSArray.isEqualToArray_)
        self.assertResultIsBOOL(Foundation.NSArray.containsObject_)
        self.assertResultIsBOOL(Foundation.NSArray.writeToFile_atomically_)
        self.assertArgIsBOOL(Foundation.NSArray.writeToFile_atomically_, 1)
        self.assertResultIsBOOL(Foundation.NSArray.writeToURL_atomically_)
        self.assertArgIsBOOL(Foundation.NSArray.writeToURL_atomically_, 1)

        self.assertArgIsSEL(Foundation.NSArray.makeObjectsPerformSelector_, 0, b"v@:")
        self.assertArgIsSEL(
            Foundation.NSArray.makeObjectsPerformSelector_withObject_, 0, b"v@:@"
        )

        self.assertArgIsBOOL(Foundation.NSArray.initWithArray_copyItems_, 1)

        self.assertArgIsIn(Foundation.NSArray.arrayWithObjects_count_, 0)
        self.assertArgSizeInArg(Foundation.NSArray.arrayWithObjects_count_, 0, 1)
        self.assertArgIsIn(Foundation.NSArray.initWithObjects_count_, 0)
        self.assertArgSizeInArg(Foundation.NSArray.initWithObjects_count_, 0, 1)

        self.assertArgIsIn(
            Foundation.NSMutableArray.removeObjectsFromIndices_numIndices_, 0
        )
        self.assertArgSizeInArg(
            Foundation.NSMutableArray.removeObjectsFromIndices_numIndices_, 0, 1
        )

        self.assertArgIsFunction(
            Foundation.NSArray.sortedArrayUsingFunction_context_, 0, b"l@@@", False
        )
        self.assertArgHasType(
            Foundation.NSArray.sortedArrayUsingFunction_context_, 1, b"@"
        )
        self.assertArgIsFunction(
            Foundation.NSArray.sortedArrayUsingFunction_context_hint_, 0, b"l@@@", False
        )
        self.assertArgHasType(
            Foundation.NSArray.sortedArrayUsingFunction_context_hint_, 1, b"@"
        )
        self.assertArgIsSEL(Foundation.NSArray.sortedArrayUsingSelector_, 0, b"i@:@")

        self.assertArgIsFunction(
            Foundation.NSMutableArray.sortUsingFunction_context_, 0, b"l@@@", False
        )
        self.assertArgHasType(
            Foundation.NSMutableArray.sortUsingFunction_context_, 1, b"@"
        )

        self.assertArgIsSEL(Foundation.NSMutableArray.sortUsingSelector_, 0, b"i@:@")

        self.assertIsNullTerminated(Foundation.NSArray.arrayWithObjects_)
        self.assertIsNullTerminated(Foundation.NSArray.initWithObjects_)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSArray.enumerateObjectsUsingBlock_,
            0,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.enumerateObjectsWithOptions_usingBlock_,
            1,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.enumerateObjectsAtIndexes_options_usingBlock_,
            2,
            b"v@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSArray.indexOfObjectPassingTest_,
            0,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.indexOfObjectWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.indexOfObjectAtIndexes_options_passingTest_,
            2,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(
            Foundation.NSArray.indexesOfObjectsPassingTest_,
            0,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.indexesOfObjectsWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSArray.indexesOfObjectsAtIndexes_options_passingTest_,
            2,
            objc._C_NSBOOL + b"@" + objc._C_NSUInteger + b"o^" + objc._C_NSBOOL,
        )

        self.assertArgIsBlock(Foundation.NSArray.sortedArrayUsingComparator_, 0, b"l@@")
        self.assertArgIsBlock(
            Foundation.NSArray.sortedArrayWithOptions_usingComparator_, 1, b"l@@"
        )
        self.assertArgIsBlock(
            Foundation.NSArray.indexOfObject_inSortedRange_options_usingComparator_,
            3,
            b"l@@",
        )
        self.assertArgHasType(
            Foundation.NSArray.indexOfObject_inSortedRange_options_usingComparator_,
            1,
            Foundation.NSRange.__typestr__,
        )

        self.assertArgIsBlock(
            Foundation.NSMutableArray.sortUsingComparator_, 0, objc._C_NSInteger + b"@@"
        )
        self.assertArgIsBlock(
            Foundation.NSMutableArray.sortWithOptions_usingComparator_,
            1,
            objc._C_NSInteger + b"@@",
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(Foundation.NSArray.writeToURL_error_)
        self.assertArgIsOut(Foundation.NSArray.writeToURL_error_, 1)

        self.assertArgIsOut(Foundation.NSArray.initWithContentsOfURL_error_, 1)
        self.assertArgIsOut(Foundation.NSArray.arrayWithContentsOfURL_error_, 1)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(
            Foundation.NSArray.differenceFromArray_withOptions_usingEquivalenceTest_,
            2,
            b"b@@",
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSBinarySearchingFirstEqual, 1 << 8)
        self.assertEqual(Foundation.NSBinarySearchingLastEqual, 1 << 9)
        self.assertEqual(Foundation.NSBinarySearchingInsertionIndex, 1 << 10)



import Cocoa
import PyObjCTools.FndCategories  # noqa: F401
from PyObjCTools.TestSupport import TestCase


class TestFndKitCategories(TestCase):
    # XXX: These tests don't actually test anything beyond asserting that
    # the code doesn't crash...

    def testNSAffineTransform(self):
        t = Cocoa.NSAffineTransform.alloc().init()
        t.rotateByDegrees_atPoint_(40, Cocoa.NSPoint(1, 2))

        t.rotateByRadians_atPoint_(2.5, Cocoa.NSPoint(2.5, 3.5))



import CoreFoundation
import objc
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestError(TestCase):
    @min_os_level("10.5")
    def testTypes(self):
        try:
            NSCFError = objc.lookUpClass("__NSCFError")
        except objc.error:
            NSCFError = objc.lookUpClass("NSCFError")

        self.assertIs(CoreFoundation.CFErrorRef, NSCFError)

    @min_os_level("10.5")
    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFErrorGetTypeID(), int)

    @min_os_level("10.5")
    def testCreation(self):
        userInfo = {"foo": "bar"}
        err = CoreFoundation.CFErrorCreate(
            None, CoreFoundation.kCFErrorDomainPOSIX, 42, userInfo
        )
        self.assertIsInstance(err, CoreFoundation.CFErrorRef)
        self.assertResultIsCFRetained(CoreFoundation.CFErrorCopyUserInfo)
        dct = CoreFoundation.CFErrorCopyUserInfo(err)
        self.assertEqual(dct, userInfo)

        keys = ["key1", "key2"]
        values = ["value1", "value2"]

        keys = [Foundation.NSString.stringWithString_(v) for v in keys]
        values = [Foundation.NSString.stringWithString_(v) for v in values]

        self.assertArgHasType(
            CoreFoundation.CFErrorCreateWithUserInfoKeysAndValues, 3, b"n^@"
        )
        self.assertArgSizeInArg(
            CoreFoundation.CFErrorCreateWithUserInfoKeysAndValues, 3, 5
        )
        self.assertArgHasType(
            CoreFoundation.CFErrorCreateWithUserInfoKeysAndValues, 4, b"n^@"
        )
        self.assertArgSizeInArg(
            CoreFoundation.CFErrorCreateWithUserInfoKeysAndValues, 4, 5
        )
        err = CoreFoundation.CFErrorCreateWithUserInfoKeysAndValues(
            None, CoreFoundation.kCFErrorDomainPOSIX, 42, keys, values, 2
        )
        self.assertIsInstance(err, CoreFoundation.CFErrorRef)
        dct = CoreFoundation.CFErrorCopyUserInfo(err)
        self.assertEqual(dct, {"key1": "value1", "key2": "value2"})

    @min_os_level("10.5")
    def testInspection(self):
        userInfo = {
            "foo": "bar",
            CoreFoundation.kCFErrorLocalizedFailureReasonKey: "failure reason",
            CoreFoundation.kCFErrorLocalizedRecoverySuggestionKey: "recovery suggestion",
        }
        self.assertResultIsCFRetained(CoreFoundation.CFErrorCreate)
        err = CoreFoundation.CFErrorCreate(
            None, CoreFoundation.kCFErrorDomainPOSIX, 42, userInfo
        )
        self.assertIsInstance(err, CoreFoundation.CFErrorRef)
        dom = CoreFoundation.CFErrorGetDomain(err)
        self.assertEqual(dom, CoreFoundation.kCFErrorDomainPOSIX)

        code = CoreFoundation.CFErrorGetCode(err)
        self.assertEqual(code, 42)

        self.assertResultIsCFRetained(CoreFoundation.CFErrorCopyDescription)
        v = CoreFoundation.CFErrorCopyDescription(err)
        self.assertIn(
            v,
            (
                "Operation could not be completed. failure reason",
                f"The operation couldn{chr(0x2019)}t be completed. failure reason",
            ),
        )

        self.assertResultIsCFRetained(CoreFoundation.CFErrorCopyFailureReason)
        v = CoreFoundation.CFErrorCopyFailureReason(err)
        self.assertEqual(v, "failure reason")

        self.assertResultIsCFRetained(CoreFoundation.CFErrorCopyRecoverySuggestion)
        v = CoreFoundation.CFErrorCopyRecoverySuggestion(err)
        self.assertEqual(v, "recovery suggestion")

    @min_os_level("10.5")
    def testConstants(self):
        self.assertIsInstance(CoreFoundation.kCFErrorDomainPOSIX, str)
        self.assertIsInstance(CoreFoundation.kCFErrorDomainOSStatus, str)
        self.assertIsInstance(CoreFoundation.kCFErrorDomainMach, str)
        self.assertIsInstance(CoreFoundation.kCFErrorDomainCocoa, str)
        self.assertIsInstance(CoreFoundation.kCFErrorLocalizedDescriptionKey, str)
        self.assertIsInstance(CoreFoundation.kCFErrorLocalizedFailureReasonKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFErrorLocalizedRecoverySuggestionKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFErrorDescriptionKey, str)
        self.assertIsInstance(CoreFoundation.kCFErrorUnderlyingErrorKey, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CoreFoundation.kCFErrorURLKey, str)
        self.assertIsInstance(CoreFoundation.kCFErrorFilePathKey, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(CoreFoundation.kCFErrorLocalizedFailureKey, str)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSComboBoxCellHelper(AppKit.NSObject):
    def numberOfItemsInComboBoxCell_(self, c):
        return 1

    def comboBoxCell_objectValueForItemAtIndex_(self, c, i):
        return 1

    def comboBoxCell_indexOfItemWithStringValue_(self, c, v):
        return 1


class TestNSComboBoxCell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSComboBoxCell.hasVerticalScroller)
        self.assertArgIsBOOL(AppKit.NSComboBoxCell.setHasVerticalScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBoxCell.isButtonBordered)
        self.assertArgIsBOOL(AppKit.NSComboBoxCell.setButtonBordered_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBoxCell.usesDataSource)
        self.assertArgIsBOOL(AppKit.NSComboBoxCell.setUsesDataSource_, 0)
        self.assertResultIsBOOL(AppKit.NSComboBoxCell.completes)
        self.assertArgIsBOOL(AppKit.NSComboBoxCell.setCompletes_, 0)

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSComboBoxCellDataSource")

    def testProtocol(self):
        self.assertResultHasType(
            TestNSComboBoxCellHelper.numberOfItemsInComboBoxCell_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSComboBoxCellHelper.comboBoxCell_objectValueForItemAtIndex_,
            1,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSComboBoxCellHelper.comboBoxCell_indexOfItemWithStringValue_,
            objc._C_NSUInteger,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSDictionaryControllerHelper(AppKit.NSObject):
    def isExplicitlyIncluded(self):
        return 1


class TestNSDictionaryController(TestCase):
    def testProtocols(self):
        self.assertResultIsBOOL(TestNSDictionaryControllerHelper.isExplicitlyIncluded)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level


class TestNSTextContentManager(TestCase):
    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertEqual(AppKit.NSTextElementProviderEnumerationOptionsNone, 0)
        self.assertEqual(AppKit.NSTextElementProviderEnumerationOptionsReverse, 1 << 0)

    @min_sdk_level("12.0")
    def testProtocols(self):
        objc.protocolNamed("NSTextElementProvider")

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertResultIsBOOL(AppKit.NSAppearance.allowsVibrancy)



import Foundation
from PyObjCTools.TestSupport import TestCase
import objc


class Behaviour(Foundation.NSObject):
    def scale(self):
        return 1

    def roundingMode(self):
        return 1

    def exceptionDuringOperation_error_leftOperand_rightOperand_(self, exc, err, x, r):
        pass


class TestNSDecimalNumber(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSDecimalNumberExactnessException, str)
        self.assertIsInstance(Foundation.NSDecimalNumberOverflowException, str)
        self.assertIsInstance(Foundation.NSDecimalNumberUnderflowException, str)
        self.assertIsInstance(Foundation.NSDecimalNumberDivideByZeroException, str)

    def testNSDecimal(self):
        dec = Foundation.NSDecimal("55.0")

        v = Foundation.NSDecimalNumber.alloc().initWithDecimal_(dec)
        self.assertIsInstance(v, Foundation.NSDecimalNumber)
        self.assertEqual(v.description(), "55")

        v = Foundation.NSDecimalNumber.decimalNumberWithDecimal_(dec)
        self.assertIsInstance(v, Foundation.NSDecimalNumber)
        self.assertEqual(v.description(), "55")

        o = v.decimalValue()
        self.assertIsInstance(o, Foundation.NSDecimal)

        o = v.objCType()
        self.assertIsInstance(o, bytes)

    def testNSNumberAsNSDecimal(self):
        v = Foundation.NSNumber.numberWithFloat_(33.5)
        o = v.decimalValue()
        self.assertIsInstance(o, Foundation.NSDecimal)

    def testNSScannerWithDecimal(self):
        v = Foundation.NSScanner.alloc().initWithString_("55.23")

        o, dec = v.scanDecimal_(None)
        self.assertIsInstance(dec, Foundation.NSDecimal)
        self.assertIs(o, True)
        self.assertEqual(str(dec), "55.23")

    def testMethods(self):
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumber.initWithMantissa_exponent_isNegative_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumber.decimalNumberWithMantissa_exponent_isNegative_, 2
        )

        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.initWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.initWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.initWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            4,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.initWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            5,
        )

        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.decimalNumberHandlerWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.decimalNumberHandlerWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.decimalNumberHandlerWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            4,
        )
        self.assertArgIsBOOL(
            Foundation.NSDecimalNumberHandler.decimalNumberHandlerWithRoundingMode_scale_raiseOnExactness_raiseOnOverflow_raiseOnUnderflow_raiseOnDivideByZero_,  # noqa: B950
            5,
        )

    def testProtocols(self):
        objc.protocolNamed("NSDecimalNumberBehaviors")
        self.assertArgHasType(
            Behaviour.exceptionDuringOperation_error_leftOperand_rightOperand_,
            0,
            objc._C_SEL,
        )
        self.assertArgHasType(
            Behaviour.exceptionDuringOperation_error_leftOperand_rightOperand_,
            1,
            objc._C_NSUInteger,
        )
        self.assertResultHasType(
            Behaviour.scale, objc._C_SHT
        )  # XXX: should this happen without a protocol definition, a bit too generic!
        self.assertResultHasType(Behaviour.roundingMode, objc._C_NSUInteger)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSAppearance(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSAppearance.allowsVibrancy)

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertArgIsBlock(
            AppKit.NSAppearance.performAsCurrentDrawingAppearance_, 0, b"v"
        )

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(AppKit.NSAppearanceNameAqua, str)
        self.assertIsInstance(AppKit.NSAppearanceNameLightContent, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(AppKit.NSAppearanceNameVibrantDark, str)
        self.assertIsInstance(AppKit.NSAppearanceNameVibrantLight, str)

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(AppKit.NSAppearanceNameDarkAqua, str)

        self.assertIsInstance(AppKit.NSAppearanceNameAccessibilityHighContrastAqua, str)
        self.assertIsInstance(
            AppKit.NSAppearanceNameAccessibilityHighContrastDarkAqua, str
        )
        self.assertIsInstance(
            AppKit.NSAppearanceNameAccessibilityHighContrastVibrantLight, str
        )
        self.assertIsInstance(
            AppKit.NSAppearanceNameAccessibilityHighContrastVibrantDark, str
        )

    @min_os_level("10.9")
    def testProtocols(self):
        objc.protocolNamed("NSAppearanceCustomization")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTitlebarAccessoryViewController(TestCase):
    @min_os_level("10.12")
    def test_methods10_12(self):
        self.assertResultIsBOOL(AppKit.NSTitlebarAccessoryViewController.isHidden)
        self.assertArgIsBOOL(AppKit.NSTitlebarAccessoryViewController.setHidden_, 0)

    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertResultIsBOOL(
            AppKit.NSTitlebarAccessoryViewController.automaticallyAdjustsSize
        )
        self.assertArgIsBOOL(
            AppKit.NSTitlebarAccessoryViewController.setAutomaticallyAdjustsSize_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSErrorHelper(AppKit.NSObject):
    def attemptRecoveryFromError_optionIndex_delegate_didRecoverSelector_contextInfo_(
        self, a, b, c, d, e
    ):
        pass

    def attemptRecoveryFromError_optionIndex_(self, a, b):
        return 1


class TestNSError(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSCocoaErrorDomain, str)

        self.assertIsInstance(AppKit.NSPOSIXErrorDomain, str)
        self.assertIsInstance(AppKit.NSOSStatusErrorDomain, str)

        self.assertIsInstance(AppKit.NSUnderlyingErrorKey, str)

        self.assertIsInstance(AppKit.NSFilePathErrorKey, str)
        self.assertIsInstance(AppKit.NSHelpAnchorErrorKey, str)
        self.assertIsInstance(AppKit.NSLocalizedDescriptionKey, str)
        self.assertIsInstance(AppKit.NSLocalizedFailureReasonErrorKey, str)
        self.assertIsInstance(AppKit.NSLocalizedRecoveryOptionsErrorKey, str)
        self.assertIsInstance(AppKit.NSLocalizedRecoverySuggestionErrorKey, str)
        self.assertIsInstance(AppKit.NSMachErrorDomain, str)
        self.assertIsInstance(AppKit.NSRecoveryAttempterErrorKey, str)
        self.assertIsInstance(AppKit.NSStringEncodingErrorKey, str)
        self.assertIsInstance(AppKit.NSURLErrorKey, str)
        self.assertIsInstance(AppKit.NSAbortModalException, str)
        self.assertIsInstance(AppKit.NSAbortPrintingException, str)
        self.assertIsInstance(AppKit.NSAccessibilityException, str)
        self.assertIsInstance(AppKit.NSAppKitIgnoredException, str)
        self.assertIsInstance(AppKit.NSAppKitVirtualMemoryException, str)
        self.assertIsInstance(AppKit.NSBadBitmapParametersException, str)
        self.assertIsInstance(AppKit.NSBadComparisonException, str)
        self.assertIsInstance(AppKit.NSBadRTFColorTableException, str)
        self.assertIsInstance(AppKit.NSBadRTFDirectiveException, str)
        self.assertIsInstance(AppKit.NSBadRTFFontTableException, str)
        self.assertIsInstance(AppKit.NSBadRTFStyleSheetException, str)
        self.assertIsInstance(AppKit.NSBrowserIllegalDelegateException, str)
        self.assertIsInstance(AppKit.NSColorListIOException, str)
        self.assertIsInstance(AppKit.NSColorListNotEditableException, str)
        self.assertIsInstance(AppKit.NSDraggingException, str)
        self.assertIsInstance(AppKit.NSFontUnavailableException, str)
        self.assertIsInstance(AppKit.NSIllegalSelectorException, str)
        self.assertIsInstance(AppKit.NSImageCacheException, str)
        self.assertIsInstance(AppKit.NSNibLoadingException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeNotFoundException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeStackOverflowException, str)
        self.assertIsInstance(AppKit.NSPPDIncludeStackUnderflowException, str)
        self.assertIsInstance(AppKit.NSPPDParseException, str)
        self.assertIsInstance(AppKit.NSPasteboardCommunicationException, str)
        self.assertIsInstance(AppKit.NSPrintPackageException, str)
        self.assertIsInstance(AppKit.NSPrintingCommunicationException, str)
        self.assertIsInstance(AppKit.NSRTFPropertyStackOverflowException, str)
        self.assertIsInstance(AppKit.NSTIFFException, str)
        self.assertIsInstance(AppKit.NSTextLineTooLongException, str)
        self.assertIsInstance(AppKit.NSTextNoSelectionException, str)
        self.assertIsInstance(AppKit.NSTextReadException, str)
        self.assertIsInstance(AppKit.NSTextWriteException, str)
        self.assertIsInstance(AppKit.NSTypedStreamVersionException, str)
        self.assertIsInstance(AppKit.NSWindowServerCommunicationException, str)
        self.assertIsInstance(AppKit.NSWordTablesReadException, str)
        self.assertIsInstance(AppKit.NSWordTablesWriteException, str)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSLocalizedFailureErrorKey, str)

    @min_os_level("11.3")
    def testConstants11_3(self):
        self.assertIsInstance(AppKit.NSMultipleUnderlyingErrorsKey, str)

    def testAttemptRecovery(self):
        self.assertArgHasType(
            TestNSErrorHelper.attemptRecoveryFromError_optionIndex_delegate_didRecoverSelector_contextInfo_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgIsSEL(
            TestNSErrorHelper.attemptRecoveryFromError_optionIndex_delegate_didRecoverSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            TestNSErrorHelper.attemptRecoveryFromError_optionIndex_delegate_didRecoverSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )

        self.assertResultIsBOOL(TestNSErrorHelper.attemptRecoveryFromError_optionIndex_)
        self.assertArgHasType(
            TestNSErrorHelper.attemptRecoveryFromError_optionIndex_,
            1,
            objc._C_NSUInteger,
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBlock(
            AppKit.NSError.setUserInfoValueProviderForDomain_provider_, 1, b"@@@"
        )
        self.assertResultIsBlock(AppKit.NSError.userInfoValueProviderForDomain_, b"@@@")
